@@ -32,6 +32,8 @@
 #include <std/Vector.h>
 #include <std/KeyedVector.h>
 
+#include "mir/fd.h"
+
 #include <linux/input.h>
 #include <sys/epoll.h>
 
@@ -194,18 +196,15 @@ public:
     virtual void setExcludedDevices(const Vector<String8>& devices) = 0;
 
     /*
-     * Wait for events to become available and returns them.
+     * Tests for available events and returns them.
      * After returning, the EventHub holds onto a wake lock until the next call to getEvent.
      * This ensures that the device will not go to sleep while the event is being processed.
      * If the device needs to remain awake longer than that, then the caller is responsible
      * for taking care of it (say, by poking the power manager user activity timer).
      *
-     * The timeout is advisory only.  If the device is asleep, it will not wake just to
-     * service the timeout.
-     *
-     * Returns the number of events obtained, or 0 if the timeout expired.
+     * Returns the number of events obtained.
      */
-    virtual size_t getEvents(int timeoutMillis, RawEvent* buffer, size_t bufferSize) = 0;
+    virtual size_t getEvents(RawEvent* buffer, size_t bufferSize) = 0;
 
     /*
      * Query current input state.
@@ -242,6 +241,9 @@ public:
     /* Wakes up getEvents() if it is blocked on a read. */
     virtual void wake() = 0;
 
+    /* Ensures that fd() readable after given timeout */
+    virtual void wakeIn(int32_t timeout) = 0;
+
     /* Dump EventHub state to a string. */
     virtual void dump(String8& dump) = 0;
 
@@ -250,6 +252,9 @@ public:
 
     /* Flush all pending events not yet read from the input devices */
     virtual void flush() = 0;
+
+    /* Epoll fd used by EventHub */
+    virtual mir::Fd fd() = 0;
 };
 
 class EventHub : public EventHubInterface
@@ -286,7 +291,7 @@ public:
     virtual bool markSupportedKeyCodes(int32_t deviceId, size_t numCodes,
             const int32_t* keyCodes, uint8_t* outFlags) const;
 
-    virtual size_t getEvents(int timeoutMillis, RawEvent* buffer, size_t bufferSize);
+    virtual size_t getEvents(RawEvent* buffer, size_t bufferSize);
 
     virtual bool hasScanCode(int32_t deviceId, int32_t scanCode) const;
     virtual bool hasLed(int32_t deviceId, int32_t led) const;
@@ -304,13 +309,19 @@ public:
     virtual void requestReopenDevices();
 
     virtual void wake();
+    virtual void wakeIn(int32_t timeout);
 
     virtual void dump(String8& dump);
     virtual void monitor();
     virtual void flush();
+    virtual mir::Fd fd();
 
     virtual ~EventHub();
 
+    // Ids used for epoll notifications not associated with devices.
+    static const uint32_t EPOLL_ID_UDEV = 0x80000001;
+    static const uint32_t EPOLL_ID_WAKE = 0x80000002;
+    static const uint32_t EPOLL_ID_TIMER = 0x80000003;
 private:
     std::shared_ptr<mir::input::InputReport> const input_report;
 
@@ -405,15 +416,12 @@ private:
     bool mNeedToScanDevices;
     Vector<String8> mExcludedDevices;
 
-    int mEpollFd;
+    mir::Fd mEpollFd;
+    mir::Fd mTimerFd;
     std::unique_ptr<mir::udev::Monitor> const device_listener;
     int mWakeReadPipeFd;
     int mWakeWritePipeFd;
 
-    // Ids used for epoll notifications not associated with devices.
-    static const uint32_t EPOLL_ID_UDEV = 0x80000001;
-    static const uint32_t EPOLL_ID_WAKE = 0x80000002;
-
     // Epoll FD list size hint.
     static const int EPOLL_SIZE_HINT = 8;
 
