@@ -26,7 +26,6 @@
 #include <androidfw/VelocityControl.h>
 #include <androidfw/VelocityTracker.h>
 #include <std/KeyedVector.h>
-#include <std/Thread.h>
 #include <std/Condition.h>
 #include <std/Timers.h>
 #include <std/RefBase.h>
@@ -450,19 +449,6 @@ private:
 };
 
 
-/* Reads raw events from the event hub and processes them, endlessly. */
-class InputReaderThread : public Thread {
-public:
-    InputReaderThread(std::shared_ptr<InputReaderInterface> const& reader);
-    virtual ~InputReaderThread();
-
-private:
-    std::shared_ptr<InputReaderInterface> mReader;
-
-    virtual bool threadLoop();
-};
-
-
 /* Represents the state of a single input device. */
 class InputDevice {
 public:
