@@ -43,7 +43,11 @@ static constexpr const std::chrono::nanoseconds NANOS_PER_MS = std::chrono::nano
 
 // Latency added during resampling.  A few milliseconds doesn't hurt much but
 // reduces the impact of mispredicted touch positions.
-static constexpr const std::chrono::nanoseconds RESAMPLE_LATENCY = std::chrono::nanoseconds(5 * NANOS_PER_MS);
+//static constexpr const std::chrono::nanoseconds RESAMPLE_LATENCY = std::chrono::nanoseconds(5 * NANOS_PER_MS);
+
+// Mir modification: No artificial latency please. This seems to provide
+// visibly and measurably lower input latency, with no noticeable down side...
+static constexpr const std::chrono::nanoseconds RESAMPLE_LATENCY(0);
 
 // Minimum time difference between consecutive samples before attempting to resample.
 static constexpr const std::chrono::nanoseconds RESAMPLE_MIN_DELTA = std::chrono::nanoseconds(2 * NANOS_PER_MS);
@@ -236,15 +240,16 @@ status_t InputPublisher::publishKeyEvent(
         int32_t scanCode,
         int32_t metaState,
         int32_t repeatCount,
+        uint64_t mac,
         std::chrono::nanoseconds downTime,
         std::chrono::nanoseconds eventTime) {
 #if DEBUG_TRANSPORT_ACTIONS
     ALOGD("channel '%s' publisher ~ publishKeyEvent: seq=%u, deviceId=%d, source=0x%x, "
             "action=0x%x, flags=0x%x, keyCode=%d, scanCode=%d, metaState=0x%x, repeatCount=%d,"
-            "downTime=%lld, eventTime=%lld",
+            "mac=%lu, downTime=%lld, eventTime=%lld",
             c_str(mChannel->getName()), seq,
             deviceId, source, action, flags, keyCode, scanCode, metaState, repeatCount,
-            downTime, eventTime);
+            mac, downTime, eventTime);
 #endif
 
     if (!seq) {
@@ -263,6 +268,7 @@ status_t InputPublisher::publishKeyEvent(
     msg.body.key.scanCode = scanCode;
     msg.body.key.metaState = metaState;
     msg.body.key.repeatCount = repeatCount;
+    msg.body.key.mac = mac;
     msg.body.key.downTime = downTime.count();
     msg.body.key.eventTime = eventTime.count();
     return mChannel->sendMessage(&msg);
@@ -281,6 +287,7 @@ status_t InputPublisher::publishMotionEvent(
         float yOffset,
         float xPrecision,
         float yPrecision,
+        uint64_t mac,
         std::chrono::nanoseconds downTime,
         std::chrono::nanoseconds eventTime,
         size_t pointerCount,
@@ -290,11 +297,11 @@ status_t InputPublisher::publishMotionEvent(
     ALOGD("channel '%s' publisher ~ publishMotionEvent: seq=%u, deviceId=%d, source=0x%x, "
             "action=0x%x, flags=0x%x, edgeFlags=0x%x, metaState=0x%x, buttonState=0x%x, "
             "xOffset=%f, yOffset=%f, "
-            "xPrecision=%f, yPrecision=%f, downTime=%lld, eventTime=%lld, "
-            "pointerCount=%d",
+            "xPrecision=%f, yPrecision=%f, mac=%lu, "
+            "downTime=%lld, eventTime=%lld, pointerCount=%d",
             c_str(mChannel->getName()), seq,
             deviceId, source, action, flags, edgeFlags, metaState, buttonState,
-            xOffset, yOffset, xPrecision, yPrecision, downTime, eventTime, pointerCount);
+            xOffset, yOffset, xPrecision, yPrecision, mac, downTime, eventTime, pointerCount);
 #endif
 
     if (!seq) {
@@ -322,6 +329,7 @@ status_t InputPublisher::publishMotionEvent(
     msg.body.motion.yOffset = yOffset;
     msg.body.motion.xPrecision = xPrecision;
     msg.body.motion.yPrecision = yPrecision;
+    msg.body.motion.mac = mac;
     msg.body.motion.downTime = downTime.count();
     msg.body.motion.eventTime = eventTime.count();
     msg.body.motion.pointerCount = pointerCount;
@@ -462,8 +470,9 @@ status_t InputConsumer::consume(InputEventFactoryInterface* factory,
             }
 
             // Start a new batch if needed.
-            if (mMsg.body.motion.action == AMOTION_EVENT_ACTION_MOVE
-                    || mMsg.body.motion.action == AMOTION_EVENT_ACTION_HOVER_MOVE) {
+            if ((mMsg.body.motion.action == AMOTION_EVENT_ACTION_MOVE
+                 || mMsg.body.motion.action == AMOTION_EVENT_ACTION_HOVER_MOVE)
+                && frameTime.count() >= 0) {
                 mBatches.push();
                 Batch& batch = mBatches.editTop();
                 batch.samples.push(mMsg);
@@ -877,6 +886,7 @@ void InputConsumer::initializeKeyEvent(KeyEvent* event, const InputMessage* msg)
             msg->body.key.scanCode,
             msg->body.key.metaState,
             msg->body.key.repeatCount,
+            msg->body.key.mac,
             std::chrono::nanoseconds(msg->body.key.downTime),
             std::chrono::nanoseconds(msg->body.key.eventTime));
 }
@@ -902,6 +912,7 @@ void InputConsumer::initializeMotionEvent(MotionEvent* event, const InputMessage
             msg->body.motion.yOffset,
             msg->body.motion.xPrecision,
             msg->body.motion.yPrecision,
+            msg->body.motion.mac,
             std::chrono::nanoseconds(msg->body.motion.downTime),
             std::chrono::nanoseconds(msg->body.motion.eventTime),
             pointerCount,
