@@ -713,7 +713,12 @@ _XcursorBuildFullname (const char *dir, const char *subdir, const char *file)
     if (!dir || !subdir || !file)
         return NULL;
 
-    full = malloc (strlen (dir) + 1 + strlen (subdir) + 1 + strlen (file) + 1);
+    /*
+     * Following the g++5 transition the strlen() in _XcursorAddPathElt() can
+     * trigger valgrind. We add some padding as a workaround.
+     */
+    size_t padding = 4;
+    full = malloc (padding + strlen (dir) + 1 + strlen (subdir) + 1 + strlen (file) + 1);
     if (!full)
 	return NULL;
     full[0] = '\0';
