@@ -16,8 +16,6 @@
  * Author: Daniel van Vugt <daniel.van.vugt@canonical.com>
  */
 
-#define MIR_INCLUDE_DEPRECATED_EVENT_HEADER
-
 #include "eglapp.h"
 #include "mir_toolkit/mir_client_library.h"
 #include <stdio.h>
@@ -96,10 +94,10 @@ static void mir_eglapp_handle_input_event(MirInputEvent const* event)
 {
     if (mir_input_event_get_type(event) != mir_input_event_type_key)
         return;
-    MirKeyInputEvent const* kev = mir_input_event_get_key_input_event(event);
-    if (mir_key_input_event_get_action(kev) != mir_key_input_event_action_up)
+    MirKeyboardEvent const* kev = mir_input_event_get_keyboard_event(event);
+    if (mir_keyboard_event_action(kev) != mir_keyboard_action_up)
         return;
-    if (mir_key_input_event_get_key_code(kev) != XKB_KEY_q)
+    if (mir_keyboard_event_key_code(kev) != XKB_KEY_q)
         return;
     
     running = 0;
@@ -144,7 +142,12 @@ static void mir_eglapp_handle_event(MirSurface* surface, MirEvent const* ev, voi
          * support for event queuing (directing them to another thread) or
          * full single-threaded callbacks. (LP: #1194384).
          */
-        printf("Resized to %dx%d\n", ev->resize.width, ev->resize.height);
+        {
+            MirResizeEvent const* resize = mir_event_get_resize_event(ev);
+            printf("Resized to %dx%d\n",
+                   mir_resize_event_get_width(resize),
+                   mir_resize_event_get_height(resize));
+        }
         break;
     case mir_event_type_close_surface:
         printf("Received close event from server.\n");
@@ -186,11 +189,6 @@ mir_eglapp_bool mir_eglapp_init(int argc, char *argv[],
         EGL_CONTEXT_CLIENT_VERSION, 2,
         EGL_NONE
     };
-    MirEventDelegate delegate = 
-    {
-        mir_eglapp_handle_event,
-        NULL
-    };
     EGLConfig eglconfig;
     EGLint neglconfigs;
     EGLContext eglctx;
@@ -199,6 +197,7 @@ mir_eglapp_bool mir_eglapp_init(int argc, char *argv[],
     unsigned int output_id = mir_display_output_id_invalid;
     char *mir_socket = NULL;
     char const* cursor_name = mir_default_cursor_name;
+    unsigned int rgb_bits = 8;
 
     if (argc > 1)
     {
@@ -210,6 +209,9 @@ mir_eglapp_bool mir_eglapp_init(int argc, char *argv[],
 
             if (arg[0] == '-')
             {
+                if (arg[1] == '-' && arg[2] == '\0')
+                    break;
+
                 switch (arg[1])
                 {
                 case 'b':
@@ -232,6 +234,21 @@ mir_eglapp_bool mir_eglapp_init(int argc, char *argv[],
                         }
                     }
                     break;
+                case 'e':
+                    {
+                        arg += 2;
+                        if (!arg[0] && i < argc-1)
+                        {
+                            ++i;
+                            arg = argv[i];
+                        }
+                        if (sscanf(arg, "%u", &rgb_bits) != 1)
+                        {
+                            printf("Invalid colour channel depth: %s\n", arg);
+                            help = 1;
+                        }
+                    }
+                    break;
                 case 'n':
                     swapinterval = 0;
                     break;
@@ -307,6 +324,7 @@ mir_eglapp_bool mir_eglapp_init(int argc, char *argv[],
             {
                 printf("Usage: %s [<options>]\n"
                        "  -b               Background opacity (0.0 - 1.0)\n"
+                       "  -e               EGL colour channel size in bits\n"
                        "  -h               Show this help text\n"
                        "  -f               Force full screen\n"
                        "  -o ID            Force placement on output monitor ID\n"
@@ -315,6 +333,7 @@ mir_eglapp_bool mir_eglapp_init(int argc, char *argv[],
                        "  -s WIDTHxHEIGHT  Force surface size\n"
                        "  -c name          Request cursor image by name\n"
                        "  -q               Quiet mode (no messages output)\n"
+                       "  --               Ignore all arguments that follow\n"
                        , argv[0]);
                 return 0;
             }
@@ -324,6 +343,49 @@ mir_eglapp_bool mir_eglapp_init(int argc, char *argv[],
     connection = mir_connect_sync(mir_socket, appname);
     CHECK(mir_connection_is_valid(connection), "Can't get connection");
 
+    egldisplay = eglGetDisplay(
+                    mir_connection_get_egl_native_display(connection));
+    CHECK(egldisplay != EGL_NO_DISPLAY, "Can't eglGetDisplay");
+
+    ok = eglInitialize(egldisplay, NULL, NULL);
+    CHECK(ok, "Can't eglInitialize");
+
+    EGLint alpha_bits = mir_eglapp_background_opacity == 1.0f ? 0 : rgb_bits;
+    const EGLint attribs[] =
+    {
+        EGL_SURFACE_TYPE, EGL_WINDOW_BIT,
+        EGL_RENDERABLE_TYPE, EGL_OPENGL_ES2_BIT,
+        EGL_RED_SIZE, rgb_bits,
+        EGL_GREEN_SIZE, rgb_bits,
+        EGL_BLUE_SIZE, rgb_bits,
+        EGL_ALPHA_SIZE, alpha_bits,
+        EGL_NONE
+    };
+
+    ok = eglChooseConfig(egldisplay, attribs, &eglconfig, 1, &neglconfigs);
+    CHECK(ok, "Could not eglChooseConfig");
+    CHECK(neglconfigs > 0, "No EGL config available");
+
+    MirPixelFormat pixel_format =
+        mir_connection_get_egl_pixel_format(connection, egldisplay, eglconfig);
+
+    printf("Mir chose pixel format %d.\n", pixel_format);
+    if (alpha_bits == 0)
+    {
+        /*
+         * If we are opaque then it's OK to switch pixel format slightly,
+         * to enable bypass/overlays to work. Otherwise the presence of an
+         * alpha channel would prevent them from being used.
+         * It would be really nice if Mesa just gave us the right answer in
+         * the first place though. (LP: #1480755)
+         */
+        if (pixel_format == mir_pixel_format_abgr_8888)
+            pixel_format = mir_pixel_format_xbgr_8888;
+        else if (pixel_format == mir_pixel_format_argb_8888)
+            pixel_format = mir_pixel_format_xrgb_8888;
+    }
+    printf("Using pixel format %d.\n", pixel_format);
+
     /* eglapps are interested in the screen size, so
        use mir_connection_create_display_config */
     MirDisplayConfiguration* display_config =
@@ -339,27 +401,6 @@ mir_eglapp_bool mir_eglapp_init(int argc, char *argv[],
 
     const MirDisplayMode *mode = &output->modes[output->current_mode];
 
-    unsigned int format[mir_pixel_formats];
-    unsigned int nformats;
-
-    mir_connection_get_available_surface_formats(connection,
-        format, mir_pixel_formats, &nformats);
-
-    MirPixelFormat pixel_format = format[0];
-    for (unsigned int f = 0; f < nformats; f++)
-    {
-        const int opaque = (format[f] == mir_pixel_format_xbgr_8888 ||
-                            format[f] == mir_pixel_format_xrgb_8888 ||
-                            format[f] == mir_pixel_format_bgr_888);
-
-        if ((mir_eglapp_background_opacity == 1.0f && opaque) ||
-            (mir_eglapp_background_opacity < 1.0f && !opaque))
-        {
-            pixel_format = format[f];
-            break;
-        }
-    }
-
     printf("Current active output is %dx%d %+d%+d\n",
            mode->horizontal_resolution, mode->vertical_resolution,
            output->position_x, output->position_y);
@@ -371,24 +412,19 @@ mir_eglapp_bool mir_eglapp_init(int argc, char *argv[],
 
     mir_display_config_destroy(display_config);
 
-    printf("Server supports %d of %d surface pixel formats. Using format: %d\n",
-        nformats, mir_pixel_formats, pixel_format);
-    unsigned int bpp = 8 * MIR_BYTES_PER_PIXEL(pixel_format);
-    EGLint attribs[] =
-    {
-        EGL_SURFACE_TYPE, EGL_WINDOW_BIT,
-        EGL_RENDERABLE_TYPE, EGL_OPENGL_ES2_BIT,
-        EGL_COLOR_BUFFER_TYPE, EGL_RGB_BUFFER,
-        EGL_BUFFER_SIZE, bpp,
-        EGL_NONE
-    };
-
     MirSurfaceSpec *spec =
         mir_connection_create_spec_for_normal_surface(connection, *width, *height, pixel_format);
 
     CHECK(spec != NULL, "Can't create a surface spec");
 
-    mir_surface_spec_set_name(spec, "eglappsurface");
+    char const* name = argv[0];
+    for (char const* p = name; *p; p++)
+    {
+        if (*p == '/')
+            name = p + 1;
+    }
+    mir_surface_spec_set_name(spec, name);
+
     if (output_id != mir_display_output_id_invalid)
         mir_surface_spec_set_fullscreen_on_output(spec, output_id);
 
@@ -397,26 +433,15 @@ mir_eglapp_bool mir_eglapp_init(int argc, char *argv[],
 
     CHECK(mir_surface_is_valid(surface), "Can't create a surface");
 
-    mir_surface_set_event_handler(surface, &delegate);
+    mir_surface_set_event_handler(surface, mir_eglapp_handle_event, NULL);
     
     MirCursorConfiguration *conf = mir_cursor_configuration_from_name(cursor_name);
     mir_surface_configure_cursor(surface, conf);
     mir_cursor_configuration_destroy(conf);
 
-    egldisplay = eglGetDisplay(
-                    mir_connection_get_egl_native_display(connection));
-    CHECK(egldisplay != EGL_NO_DISPLAY, "Can't eglGetDisplay");
-
-    ok = eglInitialize(egldisplay, NULL, NULL);
-    CHECK(ok, "Can't eglInitialize");
-
-    ok = eglChooseConfig(egldisplay, attribs, &eglconfig, 1, &neglconfigs);
-    CHECK(ok, "Could not eglChooseConfig");
-    CHECK(neglconfigs > 0, "No EGL config available");
-
     eglsurface = eglCreateWindowSurface(egldisplay, eglconfig,
-            (EGLNativeWindowType)mir_surface_get_egl_native_window(surface),
-            NULL);
+        (EGLNativeWindowType)mir_buffer_stream_get_egl_native_window(mir_surface_get_buffer_stream(surface)), NULL);
+    
     CHECK(eglsurface != EGL_NO_SURFACE, "eglCreateWindowSurface failed");
 
     eglctx = eglCreateContext(egldisplay, eglconfig, EGL_NO_CONTEXT,
