@@ -17,16 +17,19 @@
  */
 
 #include "graphics.h"
+#include "as_render_target.h"
 
 #include "mir/server.h"
 #include "mir/report_exception.h"
 #include "mir/graphics/display.h"
 #include "mir/graphics/display_buffer.h"
+#include "mir/renderer/gl/render_target.h"
 
 #include <csignal>
 
 namespace mg=mir::graphics;
 namespace mo=mir::options;
+namespace me=mir::examples;
 
 namespace
 {
@@ -53,22 +56,27 @@ void render_loop(mir::Server& server)
 
     mir::draw::glAnimationBasic gl_animation;
 
-    display->for_each_display_buffer([&](mg::DisplayBuffer& buffer)
+    display->for_each_display_sync_group([&](mg::DisplaySyncGroup& group)
     {
-        buffer.make_current();
-        gl_animation.init_gl();
+        group.for_each_display_buffer([&](mg::DisplayBuffer& buffer)
+        {
+            me::as_render_target(buffer)->make_current();
+            gl_animation.init_gl();
+        });
     });
 
     while (running)
     {
-        display->for_each_display_buffer([&](mg::DisplayBuffer& buffer)
+        display->for_each_display_sync_group([&](mg::DisplaySyncGroup& group)
         {
-            buffer.make_current();
-
-            gl_animation.render_gl();
-
-            buffer.gl_swap_buffers();
-            buffer.flip();
+            group.for_each_display_buffer([&](mg::DisplayBuffer& buffer)
+            {
+                auto const render_target = me::as_render_target(buffer);
+                render_target->make_current();
+                gl_animation.render_gl();
+                render_target->swap_buffers();
+            });
+            group.post();
         });
 
         gl_animation.step();
