@@ -37,6 +37,7 @@ class Surface;
 namespace options
 {
 class Option;
+class ProgramOption;
 }
 
 /// Graphics subsystem. Mediates interaction between core system and
@@ -49,7 +50,6 @@ class DisplayReport;
 class DisplayConfigurationPolicy;
 class GraphicBufferAllocator;
 class GLConfig;
-class GLProgramFactory;
 class PlatformIpcOperations;
 class NestedContext;
 
@@ -70,7 +70,7 @@ public:
     Platform(const Platform& p) = delete;
     Platform& operator=(const Platform& p) = delete;
 
-    virtual ~Platform() { /* TODO: make nothrow */ }
+    virtual ~Platform() = default;
 
     /**
      * Creates the buffer allocator subsystem.
@@ -82,7 +82,6 @@ public:
      */
     virtual std::shared_ptr<Display> create_display(
         std::shared_ptr<DisplayConfigurationPolicy> const& initial_conf_policy,
-        std::shared_ptr<GLProgramFactory> const& gl_program_factory,
         std::shared_ptr<GLConfig> const& gl_config) = 0;
 
     /**
@@ -105,14 +104,38 @@ public:
 enum PlatformPriority : uint32_t
 {
     unsupported = 0,    /**< Unable to function at all on this device */
+    dummy = 1,          /**< Used only for dummy or stub platforms.
+                         */
     supported = 128,    /**< Capable of providing a functioning Platform on this device,
-                         *   possibly with degraded performance or features.
+                         *    possibly with degraded performance or features.
                          */
     best = 256          /**< Capable of providing a Platform with the best features and
                          *   performance this device is capable of.
                          */
 };
 
+typedef std::shared_ptr<mir::graphics::Platform>(*CreateHostPlatform)(
+    std::shared_ptr<mir::options::Option> const& options,
+    std::shared_ptr<mir::EmergencyCleanupRegistry> const& emergency_cleanup_registry,
+    std::shared_ptr<mir::graphics::DisplayReport> const& report);
+
+typedef std::shared_ptr<mir::graphics::Platform>(*CreateGuestPlatform)(
+    std::shared_ptr<mir::graphics::DisplayReport> const& report,
+    std::shared_ptr<mir::graphics::NestedContext> const& nested_context);
+
+
+typedef void(*AddPlatformOptions)(
+    boost::program_options::options_description& config);
+
+typedef mir::graphics::PlatformPriority(*PlatformProbe)(mir::options::ProgramOption const& options);
+
+typedef mir::ModuleProperties const*(*DescribeModule)();
+}
+}
+
+extern "C"
+{
+
 /**
  * Function prototype used to return a new host graphics platform. The host graphics platform
  * is the system entity that owns the physical display and is a mir host server.
@@ -125,35 +148,28 @@ enum PlatformPriority : uint32_t
  *
  * \ingroup platform_enablement
  */
-extern "C" typedef std::shared_ptr<Platform>(*CreateHostPlatform)(
-    std::shared_ptr<options::Option> const& options,
-    std::shared_ptr<EmergencyCleanupRegistry> const& emergency_cleanup_registry,
-    std::shared_ptr<DisplayReport> const& report);
-extern "C" std::shared_ptr<Platform> create_host_platform(
-    std::shared_ptr<options::Option> const& options,
-    std::shared_ptr<EmergencyCleanupRegistry> const& emergency_cleanup_registry,
-    std::shared_ptr<DisplayReport> const& report);
+std::shared_ptr<mir::graphics::Platform> create_host_platform(
+    std::shared_ptr<mir::options::Option> const& options,
+    std::shared_ptr<mir::EmergencyCleanupRegistry> const& emergency_cleanup_registry,
+    std::shared_ptr<mir::graphics::DisplayReport> const& report);
 
 /**
  * Function prototype used to return a new guest graphics platform. The guest graphics platform
- * exists alongside the host platform and do not output or control the physical displays 
+ * exists alongside the host platform and do not output or control the physical displays
  *
- * \param [in] nested_context the object that contains resources needed from the host platform 
+ * \param [in] nested_context the object that contains resources needed from the host platform
  * \param [in] report the object to use to report interesting events from the display subsystem
  *
  * This factory function needs to be implemented by each platform.
  *
  * \ingroup platform_enablement
  */
-extern "C" typedef std::shared_ptr<Platform>(*CreateGuestPlatform)(
-    std::shared_ptr<DisplayReport> const& report,
-    std::shared_ptr<NestedContext> const& nested_context);
-extern "C" std::shared_ptr<Platform> create_guest_platform(
-    std::shared_ptr<DisplayReport> const& report,
-    std::shared_ptr<NestedContext> const& nested_context);
+std::shared_ptr<mir::graphics::Platform> create_guest_platform(
+    std::shared_ptr<mir::graphics::DisplayReport> const& report,
+    std::shared_ptr<mir::graphics::NestedContext> const& nested_context);
 
 /**
- * Function prototype used to add platform specific options to the platform-independant server options.
+ * Function prototype used to add platform specific options to the platform-independent server options.
  *
  * \param [in] config a boost::program_options that can be appended with new options
  *
@@ -161,20 +177,15 @@ extern "C" std::shared_ptr<Platform> create_guest_platform(
  *
  * \ingroup platform_enablement
  */
-extern "C" typedef void(*AddPlatformOptions)(
-    boost::program_options::options_description& config);
-extern "C" void add_graphics_platform_options(
+void add_graphics_platform_options(
     boost::program_options::options_description& config);
 
 // TODO: We actually need to be more granular here; on a device with more
 //       than one graphics system we may need a different platform per GPU,
 //       so we should be associating platforms with graphics devices in some way
-extern "C" typedef PlatformPriority(*PlatformProbe)();
-extern "C" PlatformPriority probe_graphcis_platform();
+mir::graphics::PlatformPriority probe_graphics_platform(mir::options::ProgramOption const& options);
 
-extern "C" typedef ModuleProperties const*(*DescribeModule)();
-extern "C" ModuleProperties const* describe_graphics_module();
-}
+mir::ModuleProperties const* describe_graphics_module();
 }
 
 #endif // MIR_GRAPHICS_PLATFORM_H_
