@@ -32,7 +32,7 @@ namespace frontend
 class SessionMediatorReport
 {
 public:
-    virtual ~SessionMediatorReport() { /* TODO: make nothrow */ }
+    virtual ~SessionMediatorReport() = default;
 
     virtual void session_connect_called(std::string const& app_name) = 0;
 
@@ -42,12 +42,16 @@ public:
 
     virtual void session_exchange_buffer_called(std::string const& app_name) = 0;
 
+    virtual void session_submit_buffer_called(std::string const& app_name) = 0;
+
+    virtual void session_allocate_buffers_called(std::string const& app_name) = 0;
+
+    virtual void session_release_buffers_called(std::string const& app_name) = 0;
+
     virtual void session_release_surface_called(std::string const& app_name) = 0;
 
     virtual void session_disconnect_called(std::string const& app_name) = 0;
 
-    virtual void session_drm_auth_magic_called(std::string const& app_name) = 0;
-
     virtual void session_configure_surface_called(std::string const& app_name) = 0;
 
     virtual void session_configure_surface_cursor_called(std::string const& app_name) = 0;
