@@ -15,29 +15,27 @@
  *
  * Authored by: Robert Carr <robert.carr@canonical.com>
  *              Daniel d'Andradra <daniel.dandrada@canonical.com>
+ *              Andreas Pokorny <andreas.pokorny@canonical.com>
  */
 
 #ifndef MIR_INPUT_INPUT_MANAGER_H_
 #define MIR_INPUT_INPUT_MANAGER_H_
 
-#include "mir/input/input_channel_factory.h"
-
 #include <memory>
 
 namespace mir
 {
 namespace input
 {
-class InputChannel;
+class Platform;
 
-class InputManager : public InputChannelFactory
+class InputManager
 {
 public:
+    virtual void add_platform(std::shared_ptr<Platform> const& platform) = 0;
     virtual void start() = 0;
     virtual void stop() = 0;
 
-    virtual std::shared_ptr<InputChannel> make_input_channel() = 0;
-
 protected:
     InputManager() {};
     virtual ~InputManager() {}
