@@ -28,6 +28,7 @@
 #include "mir/frontend/surface_id.h"
 #include "mir/input/input_reception_mode.h"
 #include "mir/optional_value.h"
+#include "mir/shell/surface_specification.h"
 
 #include <memory>
 #include <string>
@@ -72,6 +73,8 @@ struct SurfaceCreationParameters
 
     SurfaceCreationParameters& with_edge_attachment(MirEdgeAttachment edge);
 
+    SurfaceCreationParameters& with_buffer_stream(frontend::BufferStreamId const& id);
+
     std::string name;
     geometry::Size size;
     geometry::Point top_left;
@@ -85,10 +88,22 @@ struct SurfaceCreationParameters
     mir::optional_value<MirSurfaceType> type;
     mir::optional_value<MirOrientationMode> preferred_orientation;
     mir::optional_value<frontend::SurfaceId> parent_id;
+    mir::optional_value<frontend::BufferStreamId> content_id;
     mir::optional_value<geometry::Rectangle> aux_rect;
     mir::optional_value<MirEdgeAttachment> edge_attachment;
 
     std::weak_ptr<Surface> parent;
+
+    optional_value<geometry::Width> min_width;
+    optional_value<geometry::Height> min_height;
+    optional_value<geometry::Width> max_width;
+    optional_value<geometry::Height> max_height;
+    mir::optional_value<geometry::DeltaX> width_inc;
+    mir::optional_value<geometry::DeltaY> height_inc;
+    mir::optional_value<shell::SurfaceAspectRatio> min_aspect;
+    mir::optional_value<shell::SurfaceAspectRatio> max_aspect;
+
+    mir::optional_value<std::vector<geometry::Rectangle>> input_shape;
 };
 
 bool operator==(const SurfaceCreationParameters& lhs, const SurfaceCreationParameters& rhs);
