@@ -16,16 +16,22 @@
  * Authored by: Daniel van Vugt <daniel.van.vugt@canonical.com>
  */
 
+#define MIR_LOG_COMPONENT "DemoRenderer"
+
+#include "typo_stub_renderer.h"
+#ifdef TYPO_SUPPORTS_FREETYPE
+#include "typo_freetype_renderer.h"
+#endif
 #include "demo_renderer.h"
 #include <mir/graphics/renderable.h>
-#include <mir/compositor/destination_alpha.h>
-#include <mir/compositor/recently_used_cache.h>
+#include <mir/log.h>
 #include <cmath>
 
 using namespace mir;
 using namespace mir::examples;
 using namespace mir::geometry;
 using namespace mir::compositor;
+using namespace mir::renderer;
 
 namespace
 {
@@ -172,20 +178,17 @@ static const GLchar contrast_fshader[] =
 } // namespace
 
 DemoRenderer::DemoRenderer(
-    Rectangle const& display_area,
-    compositor::DestinationAlpha dest_alpha,
+    graphics::DisplayBuffer& display_buffer,
     float const titlebar_height,
     float const shadow_radius) :
-    GLRenderer(
-        std::unique_ptr<graphics::GLTextureCache>(new compositor::RecentlyUsedCache()),
-        display_area,
-        dest_alpha),
+    renderer::gl::Renderer(display_buffer),
     titlebar_height{titlebar_height},
     shadow_radius{shadow_radius},
     corner_radius{0.5f},
     colour_effect{none},
     inverse_program(family.add_program(vshader, inverse_fshader)),
-    contrast_program(family.add_program(vshader, contrast_fshader))
+    contrast_program(family.add_program(vshader, contrast_fshader)),
+    title_cache(std::make_shared<typo::StubRenderer>())
 {
     shadow_corner_tex = generate_shadow_corner_texture(0.4f);
     titlebar_corner_tex = generate_frame_corner_texture(corner_radius,
@@ -194,6 +197,15 @@ DemoRenderer::DemoRenderer(
 
     clear_color[0] = clear_color[1] = clear_color[2] = 0.2f;
     clear_color[3] = 1.0f;
+
+#ifdef TYPO_SUPPORTS_FREETYPE
+    const char title_font_path[] = "/usr/share/fonts/truetype/liberation/LiberationSans-Bold.ttf";
+    auto ftrenderer = std::make_shared<typo::FreetypeRenderer>();
+    if (ftrenderer->load(title_font_path, 128))
+        title_cache.change_renderer(ftrenderer);
+    else
+        mir::log_error("Failed to load titlebar font: %s", title_font_path);
+#endif
 }
 
 DemoRenderer::~DemoRenderer()
@@ -202,23 +214,31 @@ DemoRenderer::~DemoRenderer()
     glDeleteTextures(1, &titlebar_corner_tex);
 }
 
-void DemoRenderer::begin(std::unordered_set<graphics::Renderable::ID> decoration_skip_list_) const
+void DemoRenderer::begin(DecorMap&& d) const
 {
-    decoration_skip_list = decoration_skip_list_;
+    decor_map = std::move(d);
+    title_cache.drop_unused();
+    title_cache.mark_all_unused();
 }
 
-void DemoRenderer::tessellate(std::vector<graphics::GLPrimitive>& primitives,
+void DemoRenderer::tessellate(std::vector<gl::Primitive>& primitives,
                               graphics::Renderable const& renderable) const
 {
-    GLRenderer::tessellate(primitives, renderable);
-    if (decoration_skip_list.find(renderable.id()) == decoration_skip_list.end())
+    renderer::gl::Renderer::tessellate(primitives, renderable);
+    auto d = decor_map.find(renderable.id());
+    if (d != decor_map.end())
     {
-        tessellate_shadow(primitives, renderable, shadow_radius);
-        tessellate_frame(primitives, renderable, titlebar_height);
+        auto& decor = d->second;
+        if (decor->type != Decoration::Type::none)
+        {
+            tessellate_shadow(primitives, renderable, shadow_radius);
+            tessellate_frame(primitives, renderable, titlebar_height,
+                             decor->name.c_str());
+        }
     }
 }
 
-void DemoRenderer::tessellate_shadow(std::vector<graphics::GLPrimitive>& primitives,
+void DemoRenderer::tessellate_shadow(std::vector<gl::Primitive>& primitives,
                                      graphics::Renderable const& renderable,
                                      float radius) const
 {
@@ -297,9 +317,10 @@ void DemoRenderer::tessellate_shadow(std::vector<graphics::GLPrimitive>& primiti
     glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
 }
 
-void DemoRenderer::tessellate_frame(std::vector<graphics::GLPrimitive>& primitives,
+void DemoRenderer::tessellate_frame(std::vector<gl::Primitive>& primitives,
                                     graphics::Renderable const& renderable,
-                                    float titlebar_height) const
+                                    float titlebar_height,
+                                    char const* name) const
 {
     auto const& rect = renderable.screen_position();
     GLfloat left = rect.top_left.x.as_int();
@@ -307,7 +328,7 @@ void DemoRenderer::tessellate_frame(std::vector<graphics::GLPrimitive>& primitiv
     GLfloat top = rect.top_left.y.as_int();
 
     auto n = primitives.size();
-    primitives.resize(n + 3);
+    primitives.resize(n + 4);
 
     GLfloat htop = top - titlebar_height;
     GLfloat in = titlebar_height * corner_radius;
@@ -338,34 +359,28 @@ void DemoRenderer::tessellate_frame(std::vector<graphics::GLPrimitive>& primitiv
     titlebar.vertices[1] = {{inright, htop, 0.0f}, {1.0f, 0.0f}};
     titlebar.vertices[2] = {{inright, top,  0.0f}, {1.0f, 1.0f}};
     titlebar.vertices[3] = {{inleft,  top,  0.0f}, {1.0f, 1.0f}};
-}
 
-bool DemoRenderer::would_embellish(
-    graphics::Renderable const& renderable,
-    Rectangle const& display_area) const
-{
-    auto const& window = renderable.screen_position();
-    Height const full_height{2*shadow_radius + titlebar_height + window.size.height.as_int()}; 
-    Width const side_trim{shadow_radius};
-    Y const topmost_y{window.top_left.y.as_int() - shadow_radius - titlebar_height};
-    X const leftmost_x{window.top_left.x.as_int() - shadow_radius};
-    Rectangle const left{
-        Point{leftmost_x, topmost_y},
-        Size{side_trim, full_height}};
-    Rectangle const right{
-        Point{window.top_right().x, topmost_y},
-        Size{side_trim, full_height}};
-    Rectangle const bottom{
-        window.bottom_left(),
-        Size{window.size.width.as_int(), shadow_radius}};
-    Rectangle const top{
-        Point{window.top_left.x, topmost_y},
-        Size{window.size.width.as_int(), shadow_radius + titlebar_height}};
+    auto str = title_cache.get(name);
+    GLfloat text_vin = titlebar_height / 5;
+    GLfloat text_top = htop + text_vin;
+    GLfloat text_bot = top - text_vin;
+    GLfloat text_height = text_bot - text_top;
+    GLfloat text_scale = text_height / str.height;
+    GLfloat text_left = inleft;
+    GLfloat text_right = text_left + text_scale * str.width;
+    GLfloat text_u = 1.0f;
+    if (text_right > inright)  // Title too long for window
+    {
+        text_u = (inright - text_left) / (text_right - text_left);
+        text_right = inright;
+    }
 
-    return (display_area.overlaps(left) ||
-            display_area.overlaps(right) ||
-            display_area.overlaps(top) ||
-            display_area.overlaps(bottom));
+    auto& text_prim = primitives[n++];
+    text_prim.tex_id = str.tex;
+    text_prim.vertices[0] = {{text_left,  text_top, 0.0f}, {0.0f, 0.0f}};
+    text_prim.vertices[1] = {{text_right, text_top, 0.0f}, {text_u, 0.0f}};
+    text_prim.vertices[2] = {{text_right, text_bot, 0.0f}, {text_u, 1.0f}};
+    text_prim.vertices[3] = {{text_left,  text_bot, 0.0f}, {0.0f, 1.0f}};
 }
 
 void DemoRenderer::set_colour_effect(ColourEffect e)
@@ -374,14 +389,14 @@ void DemoRenderer::set_colour_effect(ColourEffect e)
 }
 
 void DemoRenderer::draw(graphics::Renderable const& renderable,
-                        GLRenderer::Program const& current_program) const
+                        renderer::gl::Renderer::Program const& current_program) const
 {
-    const GLRenderer::Program* const programs[ColourEffect::neffects] =
+    const renderer::gl::Renderer::Program* const programs[ColourEffect::neffects] =
     {
         &current_program,
         &inverse_program,
         &contrast_program
     };
 
-    GLRenderer::draw(renderable, *programs[colour_effect]);
+    renderer::gl::Renderer::draw(renderable, *programs[colour_effect]);
 }
