@@ -20,8 +20,8 @@
 #define MIR_CLIENT_CONNECTION_CONFIGURATION_H_
 
 #include <memory>
-
-namespace google { namespace protobuf { class RpcChannel; } }
+#include "lifecycle_control.h"
+#include "ping_handler.h"
 
 namespace mir
 {
@@ -41,11 +41,14 @@ class Logger;
 
 namespace client
 {
+namespace rpc
+{
+class MirBasicRpcChannel;
+}
 class ConnectionSurfaceMap;
 class Logger;
 class ClientPlatformFactory;
 class DisplayConfiguration;
-class LifecycleControl;
 class EventSink;
 class EventHandlerRegister;
 
@@ -55,12 +58,13 @@ public:
     virtual ~ConnectionConfiguration() = default;
 
     virtual std::shared_ptr<ConnectionSurfaceMap> the_surface_map() = 0;
-    virtual std::shared_ptr<google::protobuf::RpcChannel> the_rpc_channel() = 0;
+    virtual std::shared_ptr<mir::client::rpc::MirBasicRpcChannel> the_rpc_channel() = 0;
     virtual std::shared_ptr<mir::logging::Logger> the_logger() = 0;
     virtual std::shared_ptr<ClientPlatformFactory> the_client_platform_factory() = 0;
     virtual std::shared_ptr<input::receiver::InputPlatform> the_input_platform() = 0;
     virtual std::shared_ptr<DisplayConfiguration> the_display_configuration() = 0;
     virtual std::shared_ptr<LifecycleControl> the_lifecycle_control() = 0;
+    virtual std::shared_ptr<PingHandler> the_ping_handler() = 0;
     virtual std::shared_ptr<EventSink> the_event_sink() = 0;
     virtual std::shared_ptr<EventHandlerRegister> the_event_handler_register() = 0;
 
