@@ -18,41 +18,50 @@
 #ifndef MIR_CLIENT_MIR_CONNECTION_H_
 #define MIR_CLIENT_MIR_CONNECTION_H_
 
-#include <string>
-#include <memory>
-#include <unordered_set>
-#include <unordered_map>
-#include <atomic>
-
-#include <mutex>
-
-#include "mir_protobuf.pb.h"
-
-#include "mir_toolkit/mir_client_library.h"
-#include "mir_toolkit/mir_client_library_drm.h"
+#include "mir_wait_handle.h"
+#include "lifecycle_control.h"
+#include "ping_handler.h"
+#include "rpc/mir_display_server.h"
+#include "rpc/mir_display_server_debug.h"
 
+#include "mir/geometry/size.h"
 #include "mir/client_platform.h"
 #include "mir/client_context.h"
+#include "mir_toolkit/mir_client_library.h"
 
-#include "mir_wait_handle.h"
+#include <atomic>
+#include <memory>
+#include <mutex>
+#include <string>
+#include <unordered_set>
+#include <unordered_map>
 
 namespace mir
 {
 class SharedLibrary;
-
+namespace protobuf
+{
+class BufferStream;
+class Connection;
+class ConnectParameters;
+class PlatformOperationMessage;
+class DisplayConfiguration;
+}
 /// The client-side library implementation namespace
 namespace client
 {
 class ConnectionConfiguration;
 class ClientPlatformFactory;
+class ClientBufferStream;
 class ClientBufferStreamFactory;
 class ConnectionSurfaceMap;
 class DisplayConfiguration;
-class LifecycleControl;
 class EventHandlerRegister;
 
 namespace rpc
 {
+class DisplayServer;
+class DisplayServerDebug;
 class MirBasicRpcChannel;
 }
 }
@@ -72,7 +81,7 @@ class Logger;
 
 namespace dispatch
 {
-class SimpleDispatchThread;
+class ThreadedDispatcher;
 }
 }
 
@@ -113,6 +122,9 @@ public:
 
     void register_lifecycle_event_callback(mir_lifecycle_event_callback callback, void* context);
 
+    void register_ping_event_callback(mir_ping_event_callback callback, void* context);
+    void pong(int32_t serial);
+
     void register_display_change_callback(mir_display_config_callback callback, void* context);
 
     void populate(MirPlatformPackage& platform_package);
@@ -121,23 +133,40 @@ public:
                                    unsigned int formats_size, unsigned int& valid_formats);
 
     std::shared_ptr<mir::client::ClientPlatform> get_client_platform();
-    std::shared_ptr<mir::client::ClientBufferStreamFactory> get_client_buffer_stream_factory();
+
+    std::shared_ptr<mir::client::ClientBufferStream> make_consumer_stream(
+       mir::protobuf::BufferStream const& protobuf_bs, std::string const& surface_name, mir::geometry::Size);
+
+    mir::client::ClientBufferStream* create_client_buffer_stream(
+        int width, int height,
+        MirPixelFormat format,
+        MirBufferUsage buffer_usage,
+        mir_buffer_stream_callback callback,
+        void *context);
+    MirWaitHandle* release_buffer_stream(
+        mir::client::ClientBufferStream*,
+        mir_buffer_stream_callback callback,
+        void *context);
+
+    void release_consumer_stream(mir::client::ClientBufferStream*);
 
     static bool is_valid(MirConnection *connection);
 
     EGLNativeDisplayType egl_native_display();
+    MirPixelFormat       egl_pixel_format(EGLDisplay, EGLConfig) const;
 
     void on_surface_created(int id, MirSurface* surface);
+    void on_stream_created(int id, mir::client::ClientBufferStream* stream);
 
     MirWaitHandle* configure_display(MirDisplayConfiguration* configuration);
     void done_display_configure();
 
-    std::shared_ptr<google::protobuf::RpcChannel> rpc_channel() const
+    std::shared_ptr<mir::client::rpc::MirBasicRpcChannel> rpc_channel() const
     {
         return channel;
     }
 
-    mir::protobuf::DisplayServer& display_server();
+    mir::client::rpc::DisplayServer& display_server();
     std::shared_ptr<mir::logging::Logger> const& the_logger() const;
 
 private:
@@ -150,20 +179,20 @@ private:
     // from a shared library, e.g., the ClientPlatform* objects.
     std::shared_ptr<mir::SharedLibrary> const platform_library;
 
-    std::mutex mutex; // Protects all members of *this (except release_wait_handles)
+    mutable std::mutex mutex; // Protects all members of *this (except release_wait_handles)
 
-    std::shared_ptr<google::protobuf::RpcChannel> const channel;
-    mir::protobuf::DisplayServer::Stub server;
-    mir::protobuf::Debug::Stub debug;
+    std::shared_ptr<mir::client::rpc::MirBasicRpcChannel> const channel;
+    mir::client::rpc::DisplayServer server;
+    mir::client::rpc::DisplayServerDebug debug;
     std::shared_ptr<mir::logging::Logger> const logger;
-    mir::protobuf::Void void_response;
-    mir::protobuf::Connection connect_result;
+    std::unique_ptr<mir::protobuf::Void> void_response;
+    std::unique_ptr<mir::protobuf::Connection> connect_result;
     std::atomic<bool> connect_done;
-    mir::protobuf::Void ignored;
-    mir::protobuf::ConnectParameters connect_parameters;
-    mir::protobuf::PlatformOperationMessage platform_operation_reply;
-    mir::protobuf::DisplayConfiguration display_configuration_response;
-    bool disconnecting{false};
+    std::unique_ptr<mir::protobuf::Void> ignored;
+    std::unique_ptr<mir::protobuf::ConnectParameters> connect_parameters;
+    std::unique_ptr<mir::protobuf::PlatformOperationMessage> platform_operation_reply;
+    std::unique_ptr<mir::protobuf::DisplayConfiguration> display_configuration_response;
+    std::atomic<bool> disconnecting{false};
 
     std::shared_ptr<mir::client::ClientPlatformFactory> const client_platform_factory;
     std::shared_ptr<mir::client::ClientPlatform> platform;
@@ -185,22 +214,28 @@ private:
 
     std::shared_ptr<mir::client::LifecycleControl> const lifecycle_control;
 
+    std::shared_ptr<mir::client::PingHandler> const ping_handler;
+
     std::shared_ptr<mir::client::ConnectionSurfaceMap> const surface_map;
 
     std::shared_ptr<mir::client::EventHandlerRegister> const event_handler_register;
 
-    std::unique_ptr<mir::dispatch::SimpleDispatchThread> const eventloop;
+    std::unique_ptr<google::protobuf::Closure> const pong_callback;
+
+    std::unique_ptr<mir::dispatch::ThreadedDispatcher> const eventloop;
     
     std::shared_ptr<mir::client::ClientBufferStreamFactory> buffer_stream_factory;
 
     struct SurfaceRelease;
+    struct StreamRelease;
 
     MirConnection* next_valid{nullptr};
 
     void set_error_message(std::string const& error);
     void done_disconnect();
     void connected(mir_connected_callback callback, void * context);
-    void released(SurfaceRelease );
+    void released(SurfaceRelease);
+    void released(StreamRelease);
     void done_platform_operation(mir_platform_operation_callback, void* context);
     bool validate_user_display_config(MirDisplayConfiguration* config);
 };
