@@ -23,12 +23,33 @@
 #include "mir_protobuf_wire.pb.h"
 #include "mir/frontend/client_constants.h"
 #include "mir/variable_length_array.h"
+#include "mir/protobuf/protocol_version.h"
+#include "mir/log.h"
 
 #include <sstream>
 
 namespace mclr = mir::client::rpc;
 namespace mclrd = mir::client::rpc::detail;
 
+namespace
+{
+int get_protocol_version()
+{
+    auto protocol_version = mir::protobuf::current_protocol_version();
+
+    if (auto const protocol_version_override = getenv("MIR_CLIENT_TEST_OVERRRIDE_PROTOCOL_VERSION"))
+    {
+        int const new_protcol_version = strtol(protocol_version_override, nullptr, 0);
+        mir::log(mir::logging::Severity::warning, MIR_LOG_COMPONENT,
+                 "Overriding protocol version 0x%x with 0x%x", protocol_version, new_protcol_version);
+
+        protocol_version = new_protcol_version;
+    }
+
+    return protocol_version;
+}
+}
+
 mclrd::PendingCallCache::PendingCallCache(
     std::shared_ptr<RpcReport> const& rpc_report)
     : rpc_report{rpc_report}
@@ -37,14 +58,20 @@ mclrd::PendingCallCache::PendingCallCache(
 
 void mclrd::PendingCallCache::save_completion_details(
     mir::protobuf::wire::Invocation const& invoke,
-    google::protobuf::Message* response,
-    std::shared_ptr<google::protobuf::Closure> const& complete)
+    google::protobuf::MessageLite* response,
+    google::protobuf::Closure* complete)
 {
     std::unique_lock<std::mutex> lock(mutex);
 
     pending_calls[invoke.id()] = PendingCall(response, complete);
 }
 
+google::protobuf::MessageLite* mclrd::PendingCallCache::message_for_result(mir::protobuf::wire::Result& result)
+{
+    std::unique_lock<std::mutex> lock(mutex);
+    return pending_calls.at(result.id()).response;
+}
+
 void mclrd::PendingCallCache::complete_response(mir::protobuf::wire::Result& result)
 {
     PendingCall completion;
@@ -66,7 +93,6 @@ void mclrd::PendingCallCache::complete_response(mir::protobuf::wire::Result& res
     else
     {
         rpc_report->complete_response(result);
-        completion.response->ParseFromString(result.response());
         completion.complete->Run();
     }
 }
@@ -94,18 +120,16 @@ bool mclrd::PendingCallCache::empty() const
 
 
 mclr::MirBasicRpcChannel::MirBasicRpcChannel() :
-    next_message_id(0)
-{
-}
-
-mclr::MirBasicRpcChannel::~MirBasicRpcChannel()
+    next_message_id(0),
+    protocol_version{get_protocol_version()}
 {
 }
 
+mclr::MirBasicRpcChannel::~MirBasicRpcChannel() = default;
 
 mir::protobuf::wire::Invocation mclr::MirBasicRpcChannel::invocation_for(
-    google::protobuf::MethodDescriptor const* method,
-    google::protobuf::Message const* request,
+    std::string const& method_name,
+    google::protobuf::MessageLite const* request,
     size_t num_side_channel_fds)
 {
     mir::VariableLengthArray<mir::frontend::serialization_buffer_size>
@@ -116,9 +140,9 @@ mir::protobuf::wire::Invocation mclr::MirBasicRpcChannel::invocation_for(
     mir::protobuf::wire::Invocation invoke;
 
     invoke.set_id(next_id());
-    invoke.set_method_name(method->name());
+    invoke.set_method_name(method_name);
     invoke.set_parameters(buffer.data(), buffer.size());
-    invoke.set_protocol_version(1);
+    invoke.set_protocol_version(protocol_version);
     invoke.set_side_channel_fds(num_side_channel_fds);
 
     return invoke;
