@@ -19,13 +19,12 @@
 #ifndef MIR_INPUT_RECEIVER_XKB_MAPPER_H_
 #define MIR_INPUT_RECEIVER_XKB_MAPPER_H_
 
-#define MIR_INCLUDE_DEPRECATED_EVENT_HEADER
-
 #include "mir_toolkit/event.h"
 
 #include <xkbcommon/xkbcommon.h>
 
 #include <memory>
+#include <mutex>
 
 namespace mir
 {
@@ -40,13 +39,17 @@ public:
     XKBMapper();
     virtual ~XKBMapper() = default;
 
-    void update_state_and_map_event(MirKeyEvent& key_ev);
+    void set_rules(xkb_rule_names const& names);
+
+    void update_state_and_map_event(MirEvent& ev);
 
 protected:
     XKBMapper(XKBMapper const&) = delete;
     XKBMapper& operator=(XKBMapper const&) = delete;
 
 private:
+    std::mutex guard;
+    
     std::shared_ptr<xkb_context> context;
     std::shared_ptr<xkb_keymap> map;
     std::shared_ptr<xkb_state> state;
