@@ -1,5 +1,5 @@
 /*
- * Copyright © 2012-2014 Canonical Ltd.
+ * Copyright © 2012-2015 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 3,
@@ -24,12 +24,12 @@
 #include "mir/default_server_status_listener.h"
 #include "mir/emergency_cleanup.h"
 #include "mir/default_configuration.h"
+#include "mir/cookie_factory.h"
 
 #include "mir/logging/dumb_console_logger.h"
 #include "mir/options/program_option.h"
 #include "mir/frontend/session_credentials.h"
 #include "mir/frontend/session_authorizer.h"
-#include "mir/scene/surface_configurator.h"
 #include "mir/graphics/cursor.h"
 #include "mir/scene/null_session_listener.h"
 #include "mir/graphics/display.h"
@@ -42,6 +42,8 @@
 #include "mir/scene/null_prompt_session_listener.h"
 #include "default_emergency_cleanup.h"
 
+#include <type_traits>
+
 namespace mc = mir::compositor;
 namespace geom = mir::geometry;
 namespace mf = mir::frontend;
@@ -52,6 +54,11 @@ namespace ms = mir::scene;
 namespace msh = mir::shell;
 namespace mi = mir::input;
 
+namespace
+{
+    unsigned const secret_size{64};
+}
+
 mir::DefaultServerConfiguration::DefaultServerConfiguration(int argc, char const* argv[]) :
         DefaultServerConfiguration(std::make_shared<mo::DefaultConfiguration>(argc, argv))
 {
@@ -101,25 +108,6 @@ mir::DefaultServerConfiguration::the_prompt_session_listener()
         });
 }
 
-std::shared_ptr<ms::SurfaceConfigurator> mir::DefaultServerConfiguration::the_surface_configurator()
-{
-    struct DefaultSurfaceConfigurator : public ms::SurfaceConfigurator
-    {
-        int select_attribute_value(ms::Surface const&, MirSurfaceAttrib, int requested_value)
-        {
-            return requested_value;
-        }
-        void attribute_set(ms::Surface const&, MirSurfaceAttrib, int)
-        {
-        }
-    };
-    return surface_configurator(
-        [this]()
-        {
-            return std::make_shared<DefaultSurfaceConfigurator>();
-        });
-}
-
 std::shared_ptr<mf::SessionAuthorizer>
 mir::DefaultServerConfiguration::the_session_authorizer()
 {
@@ -193,6 +181,18 @@ std::shared_ptr<mir::EmergencyCleanup> mir::DefaultServerConfiguration::the_emer
         });
 }
 
+std::shared_ptr<mir::cookie::CookieFactory> mir::DefaultServerConfiguration::the_cookie_factory()
+{
+    return cookie_factory(
+        []()
+        {
+            static_assert(secret_size >= mir::cookie::CookieFactory::minimum_secret_size,
+                          "Secret size is smaller then the minimum secret size");
+
+            return mir::cookie::CookieFactory::create_keeping_secret();
+        });
+}
+
 auto mir::DefaultServerConfiguration::the_fatal_error_strategy()
 -> void (*)(char const* reason, ...)
 {
