@@ -18,11 +18,6 @@
  *   Thomas Voss <thomas.voss@canonical.com>
  */
 
-// TODO: Eliminate usage of std::chrono::nanoseconds in input_dispatcher.h
-// to remove this.
-#define MIR_INCLUDE_DEPRECATED_EVENT_HEADER 
-
-#define MIR_LOG_COMPONENT "DisplayServer"
 #include "mir/display_server.h"
 #include "mir/server_configuration.h"
 #include "mir/main_loop.h"
@@ -36,6 +31,7 @@
 #include "mir/input/input_manager.h"
 #include "mir/input/input_dispatcher.h"
 #include "mir/log.h"
+#include "mir/unwind_helpers.h"
 
 #include <stdexcept>
 
@@ -45,34 +41,6 @@ namespace mg = mir::graphics;
 namespace mi = mir::input;
 namespace msh = mir::shell;
 
-namespace
-{
-
-class TryButRevertIfUnwinding
-{
-public:
-    TryButRevertIfUnwinding(std::function<void()> const& apply,
-                            std::function<void()> const& revert)
-        : revert{revert}
-    {
-        apply();
-    }
-
-    ~TryButRevertIfUnwinding()
-    {
-        if (std::uncaught_exception())
-            revert();
-    }
-
-private:
-    TryButRevertIfUnwinding(TryButRevertIfUnwinding const&) = delete;
-    TryButRevertIfUnwinding& operator=(TryButRevertIfUnwinding const&) = delete;
-
-    std::function<void()> const revert;
-};
-
-}
-
 struct mir::DisplayServer::Private
 {
     Private(ServerConfiguration& config)
@@ -102,29 +70,29 @@ struct mir::DisplayServer::Private
     {
         try
         {
-            TryButRevertIfUnwinding dispatcher{
-                [this] { input_dispatcher->stop(); },
-                [this] { input_dispatcher->start(); }};
-
-            TryButRevertIfUnwinding input{
-                [this] { input_manager->stop(); },
-                [this] { input_manager->start(); }};
-
-            TryButRevertIfUnwinding display_config_processing{
-                [this] { display_changer->pause_display_config_processing(); },
-                [this] { display_changer->resume_display_config_processing(); }};
-
-            TryButRevertIfUnwinding comp{
-                [this] { compositor->stop(); },
-                [this] { compositor->start(); }};
-
-            TryButRevertIfUnwinding prompt{
-                [this] { prompt_connector->stop(); },
-                [this] { prompt_connector->start(); }};
-
-            TryButRevertIfUnwinding comm{
+            auto comm = try_but_revert_if_unwinding(
                 [this] { connector->stop(); },
-                [this] { connector->start(); }};
+                [this] { connector->start(); });
+
+            auto prompt = try_but_revert_if_unwinding(
+                [this] { prompt_connector->stop(); },
+                [this] { prompt_connector->start(); });
+
+            auto dispatcher = try_but_revert_if_unwinding(
+                [this] { input_dispatcher->stop(); },
+                [this] { input_dispatcher->start(); });
+
+            auto input = try_but_revert_if_unwinding(
+                [this] { input_manager->stop(); },
+                [this] { input_manager->start(); });
+
+            auto display_config_processing = try_but_revert_if_unwinding(
+                [this] { display_changer->pause_display_config_processing(); },
+                [this] { display_changer->resume_display_config_processing(); });
+
+            auto comp = try_but_revert_if_unwinding(
+                [this] { compositor->stop(); },
+                [this] { compositor->start(); });
 
             display->pause();
         }
@@ -142,31 +110,31 @@ struct mir::DisplayServer::Private
     {
         try
         {
-            TryButRevertIfUnwinding disp{
+            auto disp = try_but_revert_if_unwinding(
                 [this] { display->resume(); },
-                [this] { display->pause(); }};
+                [this] { display->pause(); });
 
-            TryButRevertIfUnwinding comm{
-                [this] { connector->start(); },
-                [this] { connector->stop(); }};
+            auto comp = try_but_revert_if_unwinding(
+                [this] { compositor->start(); },
+                [this] { compositor->stop(); });
 
-            TryButRevertIfUnwinding prompt{
-                [this] { prompt_connector->start(); },
-                [this] { prompt_connector->stop(); }};
-
-            TryButRevertIfUnwinding display_config_processing{
+            auto display_config_processing = try_but_revert_if_unwinding(
                 [this] { display_changer->resume_display_config_processing(); },
-                [this] { display_changer->pause_display_config_processing(); }};
+                [this] { display_changer->pause_display_config_processing(); });
 
-            TryButRevertIfUnwinding input{
+            auto input = try_but_revert_if_unwinding(
                 [this] { input_manager->start(); },
-                [this] { input_manager->stop(); }};
+                [this] { input_manager->stop(); });
 
-            TryButRevertIfUnwinding dispatcher{
+            auto dispatcher = try_but_revert_if_unwinding(
                 [this] { input_dispatcher->start(); },
-                [this] { input_dispatcher->stop(); }};
+                [this] { input_dispatcher->stop(); });
 
-            compositor->start();
+            auto prompt = try_but_revert_if_unwinding(
+                [this] { prompt_connector->start(); },
+                [this] { prompt_connector->stop(); });
+
+            connector->start();
         }
         catch(std::runtime_error const&)
         {
@@ -210,30 +178,35 @@ mir::DisplayServer::DisplayServer(ServerConfiguration& config) :
  * can define the 'p' member variable as a unique_ptr to an
  * incomplete type (DisplayServerPrivate) in the header.
  */
-mir::DisplayServer::~DisplayServer() = default;
+mir::DisplayServer::~DisplayServer()
+{
+    delete p.load();
+}
 
 void mir::DisplayServer::run()
 {
     mir::log_info("Mir version " MIR_VERSION);
 
-    p->connector->start();
-    p->prompt_connector->start();
-    p->compositor->start();
-    p->input_manager->start();
-    p->input_dispatcher->start();
+    auto const& server = *p.load();
 
-    p->server_status_listener->started();
+    server.compositor->start();
+    server.input_manager->start();
+    server.input_dispatcher->start();
+    server.prompt_connector->start();
+    server.connector->start();
 
-    p->main_loop->run();
+    server.server_status_listener->started();
 
-    p->input_dispatcher->stop();
-    p->input_manager->stop();
-    p->compositor->stop();
-    p->prompt_connector->stop();
-    p->connector->stop();
+    server.main_loop->run();
+
+    server.connector->stop();
+    server.prompt_connector->stop();
+    server.input_dispatcher->stop();
+    server.input_manager->stop();
+    server.compositor->stop();
 }
 
 void mir::DisplayServer::stop()
 {
-    p->main_loop->stop();
+    p.load()->main_loop->stop();
 }
