@@ -20,32 +20,40 @@
 #define MIR_FRONTEND_EVENT_SENDER_H_
 
 #include "mir/frontend/event_sink.h"
+#include "mir/frontend/fd_sets.h"
 #include <memory>
 
 namespace mir
 {
+namespace graphics { class PlatformIpcOperations; }
 namespace protobuf
 {
 class EventSequence;
 }
 namespace frontend
 {
+class MessageSender;
+
 namespace detail
 {
-class MessageSender;
 
 class EventSender : public  mir::frontend::EventSink
 {
 public:
-    explicit EventSender(std::shared_ptr<MessageSender> const& socket_sender);
+    explicit EventSender(
+        std::shared_ptr<MessageSender> const& socket_sender,
+        std::shared_ptr<graphics::PlatformIpcOperations> const& buffer_packer);
     void handle_event(MirEvent const& e);
     void handle_lifecycle_event(MirLifecycleState state);
     void handle_display_config_change(graphics::DisplayConfiguration const& config);
+    void send_ping(int32_t serial);
+    void send_buffer(frontend::BufferStreamId id, graphics::Buffer& buffer, graphics::BufferIpcMsgType);
 
 private:
-    void send_event_sequence(protobuf::EventSequence&);
+    void send_event_sequence(protobuf::EventSequence&, FdSets const&);
 
     std::shared_ptr<MessageSender> const sender;
+    std::shared_ptr<graphics::PlatformIpcOperations> const buffer_packer;
 };
 
 }
