@@ -22,6 +22,7 @@
 #include "mir/frontend/protobuf_message_sender.h"
 #include "mir/frontend/template_protobuf_message_processor.h"
 #include "mir/frontend/unsupported_feature_exception.h"
+#include <mir/protobuf/display_server_debug.h>
 
 #include "mir_protobuf_wire.pb.h"
 
@@ -58,6 +59,7 @@ namespace frontend
 namespace detail
 {
 template<> struct result_ptr_t<::mir::protobuf::Buffer>     { typedef ::mir::protobuf::Buffer* type; };
+template<> struct result_ptr_t<::mir::protobuf::BufferStream>     { typedef ::mir::protobuf::BufferStream* type; };
 template<> struct result_ptr_t<::mir::protobuf::Connection> { typedef ::mir::protobuf::Connection* type; };
 template<> struct result_ptr_t<::mir::protobuf::Surface>    { typedef ::mir::protobuf::Surface* type; };
 template<> struct result_ptr_t<::mir::protobuf::Screencast> { typedef ::mir::protobuf::Screencast* type; };
@@ -70,7 +72,8 @@ template<class ParameterMessage>
 ParameterMessage parse_parameter(Invocation const& invocation)
 {
     ParameterMessage request;
-    request.ParseFromString(invocation.parameters());
+    if (!request.ParseFromString(invocation.parameters()))
+        BOOST_THROW_EXCEPTION(std::runtime_error("Failed to parse message parameters!"));
     return request;
 }
 
@@ -105,7 +108,6 @@ void invoke(
     std::shared_ptr<ProtobufMessageProcessor> const& mp,
     DisplayServer* server,
     void (mir::protobuf::DisplayServer::*function)(
-        ::google::protobuf::RpcController* controller,
         const RequestType* request,
         ResponseType* response,
         ::google::protobuf::Closure* done),
@@ -129,14 +131,15 @@ void invoke(
     try
     {
         (server->*function)(
-            0,
             request,
             result_message.get(),
             callback);
     }
     catch (std::exception const& x)
     {
-        result_message->set_error(boost::diagnostic_information(x));
+        using namespace std::literals;
+        result_message->set_error("Error processing request: "s +
+            x.what() + "\nInternal error details: " + boost::diagnostic_information(x));
         callback->Run();
     }
 }
@@ -147,8 +150,7 @@ void invoke(
     Self* self,
     std::string* error,
     void (ServerX::*/*function*/)(
-        ::google::protobuf::RpcController* controller,
-        const ParameterMessage* request,
+        ParameterMessage const* request,
         ResultMessage* response,
         ::google::protobuf::Closure* done),
         Invocation const& invocation)
@@ -216,14 +218,22 @@ bool mfd::ProtobufMessageProcessor::dispatch(
                 request.mutable_buffer()->add_fd(fd);
             invoke(shared_from_this(), display_server.get(), &DisplayServer::exchange_buffer, invocation.id(), &request);
         }
+        else if ("submit_buffer" == invocation.method_name())
+        {
+            invoke(this, display_server.get(), &DisplayServer::submit_buffer, invocation);
+        }
+        else if ("allocate_buffers" == invocation.method_name())
+        {
+            invoke(this, display_server.get(), &DisplayServer::allocate_buffers, invocation);
+        }
+        else if ("release_buffers" == invocation.method_name())
+        {
+            invoke(this, display_server.get(), &DisplayServer::release_buffers, invocation);
+        }
         else if ("release_surface" == invocation.method_name())
         {
             invoke(this, display_server.get(), &DisplayServer::release_surface, invocation);
         }
-        else if ("drm_auth_magic" == invocation.method_name())
-        {
-            invoke(this, display_server.get(), &DisplayServer::drm_auth_magic, invocation);
-        }
         else if ("platform_operation" == invocation.method_name())
         {
             auto request = parse_parameter<mir::protobuf::PlatformOperationMessage>(invocation);
@@ -243,6 +253,10 @@ bool mfd::ProtobufMessageProcessor::dispatch(
         {
             invoke(this, display_server.get(), &DisplayServer::configure_surface, invocation);
         }
+        else if ("modify_surface" == invocation.method_name())
+        {
+            invoke(this, display_server.get(), &DisplayServer::modify_surface, invocation);
+        }
         else if ("create_screencast" == invocation.method_name())
         {
             invoke(this, display_server.get(), &DisplayServer::create_screencast, invocation);
@@ -255,6 +269,14 @@ bool mfd::ProtobufMessageProcessor::dispatch(
         {
             invoke(this, display_server.get(), &DisplayServer::release_screencast, invocation);
         }
+        else if ("create_buffer_stream" == invocation.method_name())
+        {
+            invoke(this, display_server.get(), &DisplayServer::create_buffer_stream, invocation);
+        }
+        else if ("release_buffer_stream" == invocation.method_name())
+        {
+            invoke(this, display_server.get(), &DisplayServer::release_buffer_stream, invocation);
+        }
         else if ("configure_cursor" == invocation.method_name())
         {
             invoke(this, display_server.get(), &protobuf::DisplayServer::configure_cursor, invocation);
@@ -276,24 +298,36 @@ bool mfd::ProtobufMessageProcessor::dispatch(
             invoke(this, display_server.get(), &DisplayServer::disconnect, invocation);
             result = false;
         }
+        else if ("pong" == invocation.method_name())
+        {
+            invoke(this, display_server.get(), &DisplayServer::pong, invocation);
+        }
+        else if ("configure_buffer_stream" == invocation.method_name())
+        {
+            invoke(this, display_server.get(), &DisplayServer::configure_buffer_stream, invocation);
+        }
         else if ("translate_surface_to_screen" == invocation.method_name())
         {
             try
             {
-                auto debug_interface = dynamic_cast<mir::protobuf::Debug*>(display_server.get());
-                invoke(this, debug_interface, &mir::protobuf::Debug::translate_surface_to_screen, invocation);
+                auto debug_interface = dynamic_cast<mir::protobuf::DisplayServerDebug*>(display_server.get());
+                invoke(this, debug_interface, &mir::protobuf::DisplayServerDebug::translate_surface_to_screen, invocation);
             }
             catch (mir::frontend::unsupported_feature const&)
             {
                 std::string message{"Server does not support the client debugging interface"};
                 invoke(this,
                        &message,
-                       &mir::protobuf::Debug::translate_surface_to_screen,
+                       &mir::protobuf::DisplayServerDebug::translate_surface_to_screen,
                        invocation);
                 std::runtime_error err{"Client attempted to use unavailable debug interface"};
                 report->exception_handled(display_server.get(), invocation.id(), err);
             }
         }
+        else if ("request_persistent_surface_id" == invocation.method_name())
+        {
+            invoke(this, display_server.get(), &protobuf::DisplayServer::request_persistent_surface_id, invocation);
+        }
         else
         {
             report->unknown_method(display_server.get(), invocation.id(), invocation.method_name());
@@ -311,7 +345,7 @@ bool mfd::ProtobufMessageProcessor::dispatch(
     return result;
 }
 
-void mfd::ProtobufMessageProcessor::send_response(::google::protobuf::uint32 id, ::google::protobuf::Message* response)
+void mfd::ProtobufMessageProcessor::send_response(::google::protobuf::uint32 id, ::google::protobuf::MessageLite* response)
 {
     sender->send_response(id, response, {});
 }
@@ -336,13 +370,22 @@ void mfd::ProtobufMessageProcessor::send_response(::google::protobuf::uint32 id,
 
 void mfd::ProtobufMessageProcessor::send_response(::google::protobuf::uint32 id, mir::protobuf::Surface* response)
 {
-    if (response->has_buffer_stream())
+    if (response->has_buffer_stream() && response->buffer_stream().has_buffer()) 
         sender->send_response(id, response,
             {extract_fds_from(response), extract_fds_from(response->mutable_buffer_stream()->mutable_buffer())});
     else
         sender->send_response(id, response, {extract_fds_from(response)});
 }
 
+void mfd::ProtobufMessageProcessor::send_response(::google::protobuf::uint32 id, mir::protobuf::BufferStream* response)
+{
+    if (response->has_buffer())
+        sender->send_response(id, response, {extract_fds_from(response->mutable_buffer())});
+    else
+        sender->send_response(id, response, {});
+
+}
+
 void mfd::ProtobufMessageProcessor::send_response(
     ::google::protobuf::uint32 id, mir::protobuf::Screencast* response)
 {
