@@ -38,9 +38,11 @@ class Message;
 
 namespace mir
 {
+class EmergencyCleanupRegistry;
+
 namespace frontend
 {
-class SessionCreator;
+class ConnectionCreator;
 class ConnectorReport;
 
 /// provides a client-side socket fd for each connection
@@ -48,25 +50,27 @@ class BasicConnector : public Connector
 {
 public:
     explicit BasicConnector(
-        std::shared_ptr<SessionCreator> const& session_creator,
+        std::shared_ptr<ConnectionCreator> const& connection_creator,
         int threads,
         std::shared_ptr<ConnectorReport> const& report);
     ~BasicConnector() noexcept;
     void start() override;
     void stop() override;
     int client_socket_fd() const override;
-    void remove_endpoint() const override;
-
+    int client_socket_fd(std::function<void(std::shared_ptr<Session> const& session)> const& connect_handler) const override;
 
 protected:
-    void create_session_for(std::shared_ptr<boost::asio::local::stream_protocol::socket> const& server_socket) const;
+    void create_session_for(
+        std::shared_ptr<boost::asio::local::stream_protocol::socket> const& server_socket,
+        std::function<void(std::shared_ptr<Session> const& session)> const& connect_handler) const;
+
     boost::asio::io_service mutable io_service;
     boost::asio::io_service::work work;
     std::shared_ptr<ConnectorReport> const report;
 
 private:
     std::vector<std::thread> io_service_threads;
-    std::shared_ptr<SessionCreator> const session_creator;
+    std::shared_ptr<ConnectionCreator> const connection_creator;
 };
 
 /// Accept connections over a published socket
@@ -75,13 +79,12 @@ class PublishedSocketConnector : public BasicConnector
 public:
     explicit PublishedSocketConnector(
         const std::string& socket_file,
-        std::shared_ptr<SessionCreator> const& session_creator,
+        std::shared_ptr<ConnectionCreator> const& connection_creator,
         int threads,
+        EmergencyCleanupRegistry& emergency_cleanup_registry,
         std::shared_ptr<ConnectorReport> const& report);
     ~PublishedSocketConnector() noexcept;
 
-    void remove_endpoint() const override;
-
 private:
     void start_accept();
     void on_new_connection(std::shared_ptr<boost::asio::local::stream_protocol::socket> const& socket,
