@@ -19,6 +19,7 @@
 #include "shell_wrapper.h"
 
 namespace mf = mir::frontend;
+namespace ms = mir::scene;
 
 std::shared_ptr<mf::Session> mf::ShellWrapper::open_session(
     pid_t client_pid,
@@ -33,12 +34,6 @@ void mf::ShellWrapper::close_session(std::shared_ptr<Session> const& session)
     wrapped->close_session(session);
 }
 
-void mf::ShellWrapper::handle_surface_created(
-    std::shared_ptr<Session> const& session)
-{
-    wrapped->handle_surface_created(session);
-}
-
 std::shared_ptr<mf::PromptSession> mf::ShellWrapper::start_prompt_session_for(
     std::shared_ptr<Session> const& session,
     scene::PromptSessionCreationParameters const& params)
@@ -59,9 +54,17 @@ void mf::ShellWrapper::stop_prompt_session(
     wrapped->stop_prompt_session(prompt_session);
 }
 
-mf::SurfaceId mf::ShellWrapper::create_surface(std::shared_ptr<Session> const& session, scene::SurfaceCreationParameters const& params)
+mf::SurfaceId mf::ShellWrapper::create_surface(
+    std::shared_ptr<Session> const& session,
+    scene::SurfaceCreationParameters const& params,
+    std::shared_ptr<EventSink> const& sink)
 {
-    return wrapped->create_surface(session, params);
+    return wrapped->create_surface(session, params, sink);
+}
+
+void mf::ShellWrapper::modify_surface(std::shared_ptr<Session> const& session, SurfaceId surface, shell::SurfaceSpecification const& modifications)
+{
+    wrapped->modify_surface(session, surface, modifications);
 }
 
 void mf::ShellWrapper::destroy_surface(std::shared_ptr<Session> const& session, SurfaceId surface)
@@ -69,6 +72,16 @@ void mf::ShellWrapper::destroy_surface(std::shared_ptr<Session> const& session,
     wrapped->destroy_surface(session, surface);
 }
 
+std::string mf::ShellWrapper::persistent_id_for(std::shared_ptr<Session> const& session, mf::SurfaceId surface)
+{
+    return wrapped->persistent_id_for(session, surface);
+}
+
+std::shared_ptr<ms::Surface> mf::ShellWrapper::surface_for_id(std::string const& serialised_id)
+{
+    return wrapped->surface_for_id(serialised_id);
+}
+
 int mf::ShellWrapper::set_surface_attribute(
     std::shared_ptr<Session> const& session,
     SurfaceId surface_id,
