@@ -20,9 +20,10 @@
 #include "mir/options/configuration.h"
 #include "mir/options/option.h"
 
-#include "default_display_configuration_policy.h"
+#include "mir/graphics/default_display_configuration_policy.h"
 #include "nested/mir_client_host_connection.h"
-#include "nested/nested_display.h"
+#include "nested/cursor.h"
+#include "nested/display.h"
 #include "offscreen/display.h"
 #include "software_cursor.h"
 
@@ -30,7 +31,6 @@
 #include "mir/graphics/platform.h"
 #include "mir/graphics/cursor.h"
 #include "mir/graphics/platform_probe.h"
-#include "program_factory.h"
 
 #include "mir/shared_library.h"
 #include "mir/shared_library_prober.h"
@@ -61,7 +61,7 @@ mir::DefaultServerConfiguration::the_display_configuration_policy()
         [this]
         {
             return wrap_display_configuration_policy(
-                std::make_shared<mg::DefaultDisplayConfigurationPolicy>());
+                std::make_shared<mg::CloneDisplayConfigurationPolicy>());
         });
 }
 
@@ -91,7 +91,7 @@ std::shared_ptr<mg::Platform> mir::DefaultServerConfiguration::the_graphics_plat
                     auto msg = "Failed to find any platform plugins in: " + path;
                     throw std::runtime_error(msg.c_str());
                 }
-                platform_library = mir::graphics::module_for_device(platforms);
+                platform_library = mir::graphics::module_for_device(platforms, dynamic_cast<mir::options::ProgramOption&>(*the_options()));
             }
             auto create_host_platform = platform_library->load_function<mg::CreateHostPlatform>(
                 "create_host_platform",
@@ -103,12 +103,11 @@ std::shared_ptr<mg::Platform> mir::DefaultServerConfiguration::the_graphics_plat
                 "describe_graphics_module",
                 MIR_SERVER_GRAPHICS_PLATFORM_VERSION);
             auto description = describe_module();
-            ml::log(ml::Severity::informational,
-                    std::string{"Selected driver: "} + description->name + " (version " +
-                    std::to_string(description->major_version) + "." +
-                    std::to_string(description->minor_version) + "." +
-                    std::to_string(description->micro_version) + ")",
-                    "Platform Loader");
+            mir::log_info("Selected driver: %s (version %d.%d.%d)",
+                          description->name,
+                          description->major_version,
+                          description->minor_version,
+                          description->micro_version);
 
             if (!the_options()->is_set(options::host_socket_opt))
                 return create_host_platform(the_options(), the_emergency_cleanup(), the_display_report());
@@ -145,18 +144,18 @@ mir::DefaultServerConfiguration::the_display()
             }
             else if (the_options()->is_set(options::host_socket_opt))
             {
-                return std::make_shared<mgn::NestedDisplay>(
+                return std::make_shared<mgn::Display>(
                     the_graphics_platform(),
                     the_host_connection(),
                     the_input_dispatcher(),
                     the_display_report(),
                     the_display_configuration_policy(),
-                    the_gl_config());
+                    the_gl_config(),
+                    the_cursor_listener());
             }
             {
                 return the_graphics_platform()->create_display(
                     the_display_configuration_policy(),
-                    the_gl_program_factory(),
                     the_gl_config());
             }
         });
@@ -168,28 +167,38 @@ mir::DefaultServerConfiguration::the_cursor()
     return cursor(
         [this]() -> std::shared_ptr<mg::Cursor>
         {
-            // We try to create a hardware cursor, if this fails we use a software cursor
-            auto hardware_cursor = the_display()->create_hardware_cursor(the_default_cursor_image());
-            if (hardware_cursor)
+            // In nested mode we delegate cursor presentation to the host
+            if (the_options()->is_set(options::host_socket_opt))
+            {
+                mir::log_info("Using nested cursor");
+                return wrap_cursor(std::make_shared<mgn::Cursor>(the_host_connection(), the_default_cursor_image()));
+            }
+
+            // Otherwise we try to create a hardware cursor
+            if (auto hardware_cursor = the_display()->create_hardware_cursor(the_default_cursor_image()))
             {
                 mir::log_info("Using hardware cursor");
-                return hardware_cursor;
+                return wrap_cursor(hardware_cursor);
             }
-            else
-            {
-                mir::log_info("Using software cursor");
 
-                auto const cursor = std::make_shared<mg::SoftwareCursor>(
-                    the_buffer_allocator(),
-                    the_input_scene());
+            // If other options fail we use a software cursor
+            mir::log_info("Using software cursor");
+            auto const cursor = wrap_cursor(std::make_shared<mg::SoftwareCursor>(
+                the_buffer_allocator(),
+                the_input_scene()));
 
-                cursor->show(*the_default_cursor_image());
+            cursor->show(*the_default_cursor_image());
 
-                return cursor;
-            }
+            return cursor;
         });
 }
 
+std::shared_ptr<mg::Cursor>
+mir::DefaultServerConfiguration::wrap_cursor(std::shared_ptr<mg::Cursor> const& wrapped)
+{
+    return wrapped;
+}
+
 auto mir::DefaultServerConfiguration::the_host_connection()
 -> std::shared_ptr<graphics::nested::HostConnection>
 {
@@ -242,13 +251,3 @@ mir::DefaultServerConfiguration::the_gl_config()
             return std::make_shared<NoGLConfig>();
         });
 }
-
-std::shared_ptr<mg::GLProgramFactory>
-mir::DefaultServerConfiguration::the_gl_program_factory()
-{
-    return gl_program_factory(
-        [this]
-        {
-            return std::make_shared<mg::ProgramFactory>();
-        });
-}
