@@ -58,6 +58,17 @@ private:
     EGLDisplay egl_display;
 };
 
+class DisplaySyncGroup : public graphics::DisplaySyncGroup
+{
+public:
+    DisplaySyncGroup(std::unique_ptr<DisplayBuffer> output);
+    void for_each_display_buffer(std::function<void(DisplayBuffer&)> const&) override;
+    void post() override;
+    std::chrono::milliseconds recommended_sleep() const override;
+private:
+    std::unique_ptr<DisplayBuffer> const output;
+};
+
 }
 
 class Display : public graphics::Display
@@ -68,7 +79,7 @@ public:
             std::shared_ptr<DisplayReport> const& listener);
     ~Display() noexcept;
 
-    void for_each_display_buffer(std::function<void(DisplayBuffer&)> const& f) override;
+    void for_each_display_sync_group(std::function<void(DisplaySyncGroup&)> const& f) override;
 
     std::unique_ptr<graphics::DisplayConfiguration> configuration() const override;
     void configure(graphics::DisplayConfiguration const& conf) override;
@@ -93,7 +104,7 @@ private:
     SurfacelessEGLContext const egl_context_shared;
     mutable std::mutex configuration_mutex;
     DisplayConfiguration current_display_configuration;
-    std::vector<std::unique_ptr<DisplayBuffer>> display_buffers;
+    std::vector<std::unique_ptr<DisplaySyncGroup>> display_sync_groups;
 };
 
 }
