@@ -21,8 +21,6 @@
 
 #include "mir/shell/input_targeter.h"
 
-#include <utils/StrongPointer.h>
-
 #include <mutex>
 
 namespace android
@@ -45,19 +43,19 @@ class WindowHandleRepository;
 class InputTargeter : public shell::InputTargeter
 {
 public:
-    explicit InputTargeter(droidinput::sp<droidinput::InputDispatcherInterface> const& input_dispatcher,
+    explicit InputTargeter(std::shared_ptr<droidinput::InputDispatcherInterface> const& input_dispatcher,
                            std::shared_ptr<WindowHandleRepository> const& repository);
     virtual ~InputTargeter() noexcept(true);
 
-    void focus_changed(std::shared_ptr<input::InputChannel const> const& focus_channel);
-    void focus_cleared();
+    void set_focus(std::shared_ptr<input::Surface> const& focus_surface);
+    void clear_focus();
 
 protected:
     InputTargeter(const InputTargeter&) = delete;
     InputTargeter& operator=(const InputTargeter&) = delete;
 
 private:
-    droidinput::sp<droidinput::InputDispatcherInterface> input_dispatcher;
+    std::shared_ptr<droidinput::InputDispatcherInterface> input_dispatcher;
 
     std::shared_ptr<WindowHandleRepository> const repository;
 };
