@@ -37,24 +37,35 @@ geom::Rectangle mi::DisplayInputRegion::bounding_rectangle()
 {
     geom::Rectangles rectangles;
 
-    display->for_each_display_buffer(
-        [&rectangles](mg::DisplayBuffer const& buffer)
-        {
-            rectangles.add(buffer.view_area());
-        });
+    display->for_each_display_sync_group([&rectangles](mg::DisplaySyncGroup& group)
+    {
+        group.for_each_display_buffer(
+            [&rectangles](mg::DisplayBuffer const& buffer)
+            {
+                rectangles.add(buffer.view_area());
+            });
+    });
 
-    return rectangles.bounding_rectangle();
+    //TODO: This region is mainly used for scaling touchscreen coordinates, so the caller
+    // probably wants the full list of rectangles. Additional work is needed
+    // to group a touchscreen with a display. So for now, just return the view area
+    // of the first display, as that matches the most common systems (laptops with touchscreens,
+    // phone/tablets with touchscreens).
+    return *rectangles.begin();
 }
 
 void mi::DisplayInputRegion::confine(geom::Point& point)
 {
     geom::Rectangles rectangles;
 
-    display->for_each_display_buffer(
-        [&rectangles](mg::DisplayBuffer const& buffer)
-        {
-            rectangles.add(buffer.view_area());
-        });
+    display->for_each_display_sync_group([&rectangles](mg::DisplaySyncGroup& group)
+    {
+        group.for_each_display_buffer(
+            [&rectangles](mg::DisplayBuffer const& buffer)
+            {
+                rectangles.add(buffer.view_area());
+            });
+    });
 
     rectangles.confine(point);
 }
