@@ -34,9 +34,11 @@ MIR_SESSION_MEDIATOR_EVENT_METHOD(session_connect_called)
 MIR_SESSION_MEDIATOR_EVENT_METHOD(session_create_surface_called)
 MIR_SESSION_MEDIATOR_EVENT_METHOD(session_next_buffer_called)
 MIR_SESSION_MEDIATOR_EVENT_METHOD(session_exchange_buffer_called)
+MIR_SESSION_MEDIATOR_EVENT_METHOD(session_submit_buffer_called)
+MIR_SESSION_MEDIATOR_EVENT_METHOD(session_allocate_buffers_called)
+MIR_SESSION_MEDIATOR_EVENT_METHOD(session_release_buffers_called)
 MIR_SESSION_MEDIATOR_EVENT_METHOD(session_release_surface_called)
 MIR_SESSION_MEDIATOR_EVENT_METHOD(session_disconnect_called)
-MIR_SESSION_MEDIATOR_EVENT_METHOD(session_drm_auth_magic_called)
 MIR_SESSION_MEDIATOR_EVENT_METHOD(session_configure_surface_called)
 MIR_SESSION_MEDIATOR_EVENT_METHOD(session_configure_surface_cursor_called)
 MIR_SESSION_MEDIATOR_EVENT_METHOD(session_configure_display_called)
