@@ -43,12 +43,16 @@ class SessionMediatorReport : public frontend::SessionMediatorReport
 
     void session_exchange_buffer_called(std::string const& app_name) override;
 
+    void session_submit_buffer_called(std::string const& app_name) override;
+
+    void session_allocate_buffers_called(std::string const& app_name) override;
+
+    void session_release_buffers_called(std::string const& app_name) override;
+
     void session_release_surface_called(std::string const& app_name) override;
 
     void session_disconnect_called(std::string const& app_name) override;
 
-    void session_drm_auth_magic_called(std::string const& app_name) override;
-
     void session_configure_surface_called(std::string const& app_name) override;
 
     void session_configure_surface_cursor_called(std::string const& app_name) override;
