@@ -21,7 +21,8 @@
 
 #include "mir/scene/surface.h"
 #include "mir/basic_observers.h"
-#include "mir/scene/surface_observer.h"
+#include "mir/scene/surface_observers.h"
+#include "mir/input/validator.h"
 
 #include "mir/geometry/rectangle.h"
 
@@ -29,6 +30,7 @@
 
 #include <glm/glm.hpp>
 #include <vector>
+#include <list>
 #include <memory>
 #include <mutex>
 #include <string>
@@ -54,25 +56,7 @@ class Surface;
 namespace scene
 {
 class SceneReport;
-
-class SurfaceObservers : public SurfaceObserver, BasicObservers<SurfaceObserver>
-{
-public:
-    using BasicObservers<SurfaceObserver>::add;
-    using BasicObservers<SurfaceObserver>::remove;
-
-    void attrib_changed(MirSurfaceAttrib attrib, int value) override;
-    void resized_to(geometry::Size const& size) override;
-    void moved_to(geometry::Point const& top_left) override;
-    void hidden_set_to(bool hide) override;
-    void frame_posted(int frames_available) override;
-    void alpha_set_to(float alpha) override;
-    void orientation_set_to(MirOrientation orientation) override;
-    void transformation_set_to(glm::mat4 const& t) override;
-    void reception_mode_set_to(input::InputReceptionMode mode) override;
-    void cursor_image_set_to(graphics::CursorImage const& image) override;
-    void client_surface_close_requested() override;
-};
+class CursorStreamImageAdapter;
 
 class BasicSurface : public Surface
 {
@@ -108,15 +92,11 @@ public:
     geometry::Size size() const override;
     geometry::Size client_size() const override;
 
-    MirPixelFormat pixel_format() const override;
-
-    std::shared_ptr<graphics::Buffer> snapshot_buffer() const;
-    void swap_buffers(graphics::Buffer* old_buffer, std::function<void(graphics::Buffer* new_buffer)> complete) override;
-    void force_requests_to_complete() override;
+    std::shared_ptr<frontend::BufferStream> primary_buffer_stream() const override;
+    void set_streams(std::list<scene::StreamInfo> const& streams) override;
 
     bool supports_input() const override;
     int client_input_fd() const override;
-    void allow_framedropping(bool) override;
     std::shared_ptr<input::InputChannel> input_channel() const override;
     input::InputReceptionMode reception_mode() const override;
     void set_reception_mode(input::InputReceptionMode mode) override;
@@ -136,23 +116,24 @@ public:
 
     bool visible() const override;
     
-    std::unique_ptr<graphics::Renderable> compositor_snapshot(void const* compositor_id) const override;
+    graphics::RenderableList generate_renderables(compositor::CompositorID id) const override;
     int buffers_ready_for_compositor(void const* compositor_id) const override;
 
-    void with_most_recent_buffer_do(
-        std::function<void(graphics::Buffer&)> const& exec) override;
-
     MirSurfaceType type() const override;
     MirSurfaceState state() const override;
-    void take_input_focus(std::shared_ptr<shell::InputTargeter> const& targeter) override;
     int configure(MirSurfaceAttrib attrib, int value) override;
-    int query(MirSurfaceAttrib attrib) override;
+    int query(MirSurfaceAttrib attrib) const override;
     void hide() override;
     void show() override;
     
     void set_cursor_image(std::shared_ptr<graphics::CursorImage> const& image) override;
     std::shared_ptr<graphics::CursorImage> cursor_image() const override;
 
+    void set_cursor_stream(std::shared_ptr<frontend::BufferStream> const& stream,
+                           geometry::Displacement const& hotspot) override;
+    void set_cursor_from_buffer(graphics::Buffer& buffer,
+                                geometry::Displacement const& hotspot);
+
     void request_client_surface_close() override;
 
     std::shared_ptr<Surface> parent() const override;
@@ -162,6 +143,10 @@ public:
 
     int dpi() const;
 
+    void set_keymap(xkb_rule_names const& rules) override;
+
+    void rename(std::string const& title) override;
+
 private:
     bool visible(std::unique_lock<std::mutex>&) const;
     MirSurfaceType set_type(MirSurfaceType t);  // Use configure() to make public changes
@@ -174,11 +159,10 @@ private:
 
     SurfaceObservers observers;
     std::mutex mutable guard;
-    std::string const surface_name;
+    std::string surface_name;
     geometry::Rectangle surface_rect;
     glm::mat4 transformation_matrix;
     float surface_alpha;
-    bool first_frame_posted;
     bool hidden;
     input::InputReceptionMode input_mode;
     const bool nonrectangular;
@@ -190,6 +174,7 @@ private:
     std::shared_ptr<SceneReport> const report;
     std::weak_ptr<Surface> const parent_;
 
+    std::list<StreamInfo> layers;
     // Surface attributes:
     MirSurfaceType type_ = mir_surface_type_normal;
     MirSurfaceState state_ = mir_surface_state_restored;
@@ -198,6 +183,10 @@ private:
     int dpi_ = 0;
     MirSurfaceVisibility visibility_ = mir_surface_visibility_exposed;
     MirOrientationMode pref_orientation_mode = mir_orientation_mode_any;
+
+    std::unique_ptr<CursorStreamImageAdapter> const cursor_stream_adapter;
+
+    input::Validator input_validator;
 };
 
 }
