@@ -18,11 +18,13 @@
 
 #include "mir/default_server_configuration.h"
 
+#include "mir/main_loop.h"
 #include "mir/graphics/display.h"
 #include "mir/graphics/gl_context.h"
 #include "mir/input/scene.h"
 #include "mir/abnormal_exit.h"
 #include "mir/scene/session.h"
+#include "mir/shell/display_configuration_controller.h"
 
 #include "broadcasting_session_event_sink.h"
 #include "default_session_container.h"
@@ -36,11 +38,18 @@
 #include "surface_stack.h"
 #include "threaded_snapshot_strategy.h"
 #include "prompt_session_manager_impl.h"
+#include "default_coordinate_translator.h"
+#include "timeout_application_not_responding_detector.h"
+#include "mir/options/program_option.h"
+#include "mir/options/default_configuration.h"
+#include "mir/graphics/display_configuration.h"
+#include "mir/frontend/display_changer.h"
 
 namespace mc = mir::compositor;
 namespace mf = mir::frontend;
 namespace mi = mir::input;
 namespace ms = mir::scene;
+namespace mg = mir::graphics;
 namespace msh = mir::shell;
 
 std::shared_ptr<ms::SurfaceStackModel>
@@ -70,10 +79,8 @@ auto mir::DefaultServerConfiguration::the_surface_factory()
         [this]()
         {
             return std::make_shared<ms::SurfaceAllocator>(
-                the_buffer_stream_factory(),
                 the_input_channel_factory(),
                 the_input_sender(),
-                the_surface_configurator(),
                 the_default_cursor_image(),
                 the_scene_report());
         });
@@ -85,21 +92,12 @@ mir::DefaultServerConfiguration::the_surface_coordinator()
     return surface_coordinator(
         [this]()
         {
-            return wrap_surface_coordinator(
-                std::make_shared<ms::SurfaceController>(
+            return std::make_shared<ms::SurfaceController>(
                     the_surface_factory(),
-                    the_placement_strategy(),
-                    the_surface_stack_model()));
+                    the_surface_stack_model());
         });
 }
 
-std::shared_ptr<ms::SurfaceCoordinator>
-mir::DefaultServerConfiguration::wrap_surface_coordinator(
-    std::shared_ptr<ms::SurfaceCoordinator> const& wrapped)
-{
-    return wrapped;
-}
-
 std::shared_ptr<ms::BroadcastingSessionEventSink>
 mir::DefaultServerConfiguration::the_broadcasting_session_event_sink()
 {
@@ -141,7 +139,8 @@ mir::DefaultServerConfiguration::the_mediating_display_changer()
                 the_display_configuration_policy(),
                 the_session_container(),
                 the_session_event_handler_register(),
-                the_server_action_queue());
+                the_server_action_queue(),
+                the_display_configuration_report());
         });
 
 }
@@ -174,37 +173,19 @@ mir::DefaultServerConfiguration::the_session_coordinator()
     return session_coordinator(
         [this]()
         {
-            return wrap_session_coordinator(
-                std::make_shared<ms::SessionManager>(
+            return std::make_shared<ms::SessionManager>(
                     the_surface_coordinator(),
+                    the_surface_factory(),
+                    the_buffer_stream_factory(),
                     the_session_container(),
-                    the_shell_focus_setter(),
                     the_snapshot_strategy(),
                     the_session_event_sink(),
                     the_session_listener(),
-                    the_prompt_session_manager()));
+                    the_display(),
+                    the_application_not_responding_detector());
         });
 }
 
-std::shared_ptr<ms::SessionCoordinator>
-mir::DefaultServerConfiguration::wrap_session_coordinator(
-    std::shared_ptr<ms::SessionCoordinator> const& wrapped)
-{
-    return wrapped;
-}
-
-std::shared_ptr<mf::Shell>
-mir::DefaultServerConfiguration::the_frontend_shell()
-{
-    return the_session_coordinator();
-}
-
-std::shared_ptr<msh::FocusController>
-mir::DefaultServerConfiguration::the_focus_controller()
-{
-    return the_session_coordinator();
-}
-
 std::shared_ptr<ms::PixelBuffer>
 mir::DefaultServerConfiguration::the_pixel_buffer()
 {
@@ -238,3 +219,31 @@ mir::DefaultServerConfiguration::the_prompt_session_manager()
                 the_prompt_session_listener());
         });
 }
+
+std::shared_ptr<ms::CoordinateTranslator>
+mir::DefaultServerConfiguration::the_coordinate_translator()
+{
+    return coordinate_translator(
+        [this]()
+        {
+            return std::make_shared<ms::DefaultCoordinateTranslator>();
+        });
+}
+
+auto mir::DefaultServerConfiguration::the_application_not_responding_detector()
+-> std::shared_ptr<scene::ApplicationNotRespondingDetector>
+{
+    return application_not_responding_detector(
+        [this]() -> std::shared_ptr<scene::ApplicationNotRespondingDetector>
+        {
+            using namespace std::literals::chrono_literals;
+            return std::make_shared<ms::TimeoutApplicationNotRespondingDetector>(
+                *the_main_loop(), 1s);
+        });
+}
+
+std::shared_ptr<msh::DisplayConfigurationController>
+mir::DefaultServerConfiguration::the_display_configuration_controller()
+{
+    return the_mediating_display_changer();
+}
