@@ -32,3 +32,5 @@ void ms::NullSurfaceObserver::transformation_set_to(glm::mat4 const& /*t*/) {}
 void ms::NullSurfaceObserver::reception_mode_set_to(input::InputReceptionMode /*mode*/) {}
 void ms::NullSurfaceObserver::cursor_image_set_to(mg::CursorImage const& /*image*/) {}
 void ms::NullSurfaceObserver::client_surface_close_requested() {}
+void ms::NullSurfaceObserver::keymap_changed(xkb_rule_names const& /* names */) {}
+void ms::NullSurfaceObserver::renamed(char const*) {}
