@@ -21,16 +21,15 @@
 
 #include "mir/scene/session_coordinator.h"
 
-#include <mutex>
 #include <memory>
 #include <vector>
 
 namespace mir
 {
-
-namespace shell
+namespace graphics
 {
-class FocusSetter;
+class DisplayConfiguration;
+class Display;
 }
 
 namespace scene
@@ -41,38 +40,37 @@ class SessionListener;
 class SnapshotStrategy;
 class SurfaceCoordinator;
 class PromptSessionManager;
-
+class BufferStreamFactory;
+class SurfaceFactory;
+class ApplicationNotRespondingDetector;
 
 class SessionManager : public SessionCoordinator
 {
 public:
-    explicit SessionManager(std::shared_ptr<SurfaceCoordinator> const& surface_coordinator,
-                            std::shared_ptr<SessionContainer> const& app_container,
-                            std::shared_ptr<shell::FocusSetter> const& focus_setter,
-                            std::shared_ptr<SnapshotStrategy> const& snapshot_strategy,
-                            std::shared_ptr<SessionEventSink> const& session_event_sink,
-                            std::shared_ptr<SessionListener> const& session_listener,
-                            std::shared_ptr<PromptSessionManager> const& prompt_session_manager);
+    SessionManager(
+        std::shared_ptr<SurfaceCoordinator> const& surface_coordinator,
+        std::shared_ptr<SurfaceFactory> const& surface_factory,
+        std::shared_ptr<BufferStreamFactory> const& buffer_stream_factory,
+        std::shared_ptr<SessionContainer> const& app_container,
+        std::shared_ptr<SnapshotStrategy> const& snapshot_strategy,
+        std::shared_ptr<SessionEventSink> const& session_event_sink,
+        std::shared_ptr<SessionListener> const& session_listener,
+        std::shared_ptr<graphics::Display const> const& display,
+        std::shared_ptr<ApplicationNotRespondingDetector> const& anr_detector);
+
     virtual ~SessionManager() noexcept;
 
-    virtual std::shared_ptr<frontend::Session> open_session(
+    std::shared_ptr<Session> open_session(
         pid_t client_pid,
         std::string const& name,
         std::shared_ptr<frontend::EventSink> const& sink) override;
 
-    virtual void close_session(std::shared_ptr<frontend::Session> const& session) override;
+    void close_session(std::shared_ptr<Session> const& session) override;
+
+    std::shared_ptr<Session> successor_of(std::shared_ptr<Session> const&) const override;
 
-    void focus_next() override;
-    std::weak_ptr<Session> focussed_application() const;
     void set_focus_to(std::shared_ptr<Session> const& focus) override;
-
-    void handle_surface_created(std::shared_ptr<frontend::Session> const& session) override;
-
-    std::shared_ptr<frontend::PromptSession> start_prompt_session_for(std::shared_ptr<frontend::Session> const& session,
-                                                  PromptSessionCreationParameters const& params) override;
-    void add_prompt_provider_for(std::shared_ptr<frontend::PromptSession> const& prompt_session,
-                                 std::shared_ptr<frontend::Session> const& session) override;
-    void stop_prompt_session(std::shared_ptr<frontend::PromptSession> const& prompt_session) override;
+    void unset_focus() override;
 
 protected:
     SessionManager(const SessionManager&) = delete;
@@ -80,17 +78,14 @@ protected:
 
 private:
     std::shared_ptr<SurfaceCoordinator> const surface_coordinator;
+    std::shared_ptr<SurfaceFactory> const surface_factory;
+    std::shared_ptr<BufferStreamFactory> const buffer_stream_factory;
     std::shared_ptr<SessionContainer> const app_container;
-    std::shared_ptr<shell::FocusSetter> const focus_setter;
     std::shared_ptr<SnapshotStrategy> const snapshot_strategy;
     std::shared_ptr<SessionEventSink> const session_event_sink;
     std::shared_ptr<SessionListener> const session_listener;
-    std::shared_ptr<PromptSessionManager> const prompt_session_manager;
-
-    std::mutex mutex;
-    std::weak_ptr<Session> focus_application;
-
-    void set_focus_to_locked(std::unique_lock<std::mutex> const& lock, std::shared_ptr<Session> const& next_focus);
+    std::shared_ptr<graphics::Display const> const display;
+    std::shared_ptr<ApplicationNotRespondingDetector> const anr_detector;
 };
 
 }
