@@ -98,7 +98,7 @@ int main(int argc, char *argv[])
     MirConnection *conn = mir_connect_sync(server, argv[0]);
     if (!mir_connection_is_valid(conn))
     {
-        fprintf(stderr, "Could not connect to a display server.\n");
+        fprintf(stderr, "Could not connect to a display server: %s\n", mir_connection_get_error_message(conn));
         return 1;
     }
 
