@@ -16,20 +16,24 @@
  * Authored by: Kevin DuBois <kevin.dubois@canonical.com>
  */
 
+#include "mir/compositor/display_listener.h"
+#include "mir/compositor/renderer_factory.h"
 #include "mir/scene/surface_creation_parameters.h"
 #include "src/server/report/null_report_factory.h"
 #include "src/server/scene/surface_stack.h"
-#include "src/server/compositor/gl_renderer_factory.h"
+#include "src/server/compositor/buffer_stream_surfaces.h"
 #include "src/server/scene/basic_surface.h"
 #include "src/server/compositor/default_display_buffer_compositor_factory.h"
 #include "src/server/compositor/multi_threaded_compositor.h"
-#include "mir_test/fake_shared.h"
-#include "mir_test_doubles/mock_buffer_stream.h"
-#include "mir_test_doubles/null_display.h"
-#include "mir_test_doubles/stub_renderer.h"
-#include "mir_test_doubles/stub_display_buffer.h"
-#include "mir_test_doubles/stub_buffer.h"
-#include "mir_test_doubles/stub_input_sender.h"
+#include "mir/test/fake_shared.h"
+#include "mir/test/doubles/mock_buffer_stream.h"
+#include "mir/test/doubles/mock_buffer_bundle.h"
+#include "mir/test/doubles/null_display.h"
+#include "mir/test/doubles/stub_renderer.h"
+#include "mir/test/doubles/stub_display_buffer.h"
+#include "mir/test/doubles/stub_buffer.h"
+#include "mir/test/doubles/stub_input_sender.h"
+#include "mir/test/doubles/null_display_sync_group.h"
 
 #include <condition_variable>
 #include <mutex>
@@ -49,30 +53,21 @@ namespace
 class StubRendererFactory : public mc::RendererFactory
 {
 public:
-    std::unique_ptr<mc::Renderer> create_renderer_for(geom::Rectangle const&,
-        mc::DestinationAlpha) override
+    std::unique_ptr<mc::Renderer>
+        create_renderer_for(mg::DisplayBuffer&) override
     {
         return std::unique_ptr<mtd::StubRenderer>(new mtd::StubRenderer);
     }
 };
 
-struct CountingDisplayBuffer : public mtd::StubDisplayBuffer
+struct CountingDisplaySyncGroup : public mtd::StubDisplaySyncGroup
 {
-    CountingDisplayBuffer() :
-        StubDisplayBuffer({{0,0}, {10, 10}})
+    CountingDisplaySyncGroup() :
+    mtd::StubDisplaySyncGroup({100,100})
     {
     }
 
-    bool post_renderables_if_optimizable(mg::RenderableList const&) override
-    {
-        return false;
-    }
-
-    void gl_swap_buffers() override
-    {
-    }
-
-    void flip() override
+    void post() override
     {
         increment_post_count();
     }
@@ -101,47 +96,44 @@ private:
 
 struct StubDisplay : public mtd::NullDisplay
 {
-    StubDisplay(mg::DisplayBuffer& primary, mg::DisplayBuffer& secondary)
+    StubDisplay(mg::DisplaySyncGroup& primary, mg::DisplaySyncGroup& secondary)
       : primary(primary),
         secondary(secondary)
     {
     } 
-    void for_each_display_buffer(std::function<void(mg::DisplayBuffer&)> const& fn) override
+    void for_each_display_sync_group(std::function<void(mg::DisplaySyncGroup&)> const& fn) override
     {
         fn(primary);
         fn(secondary);
     }
 private:
-    mg::DisplayBuffer& primary;
-    mg::DisplayBuffer& secondary;
+    mg::DisplaySyncGroup& primary;
+    mg::DisplaySyncGroup& secondary;
 };
 
-class BypassStubBuffer : public mtd::StubBuffer
+struct StubDisplayListener : mc::DisplayListener
 {
-public:
-    bool can_bypass() const override
-    {
-        return true;
-    }
+    virtual void add_display(geom::Rectangle const& /*area*/) override {}
+    virtual void remove_display(geom::Rectangle const& /*area*/) override {}
 };
 
 struct SurfaceStackCompositor : public testing::Test
 {
     SurfaceStackCompositor() :
         timeout{std::chrono::system_clock::now() + std::chrono::seconds(5)},
-        mock_buffer_stream(std::make_shared<testing::NiceMock<mtd::MockBufferStream>>()),
+        mock_buffer_stream(std::make_shared<testing::NiceMock<mtd::MockBufferBundle>>()),
         stub_surface{std::make_shared<ms::BasicSurface>(
             std::string("stub"),
             geom::Rectangle{{0,0},{1,1}},
             false,
-            mock_buffer_stream,
+            std::make_shared<mc::BufferStreamSurfaces>(mock_buffer_stream),
             std::shared_ptr<mir::input::InputChannel>(),
             std::shared_ptr<mtd::StubInputSender>(),
             std::shared_ptr<mg::CursorImage>(),
             null_scene_report)}
     {
         using namespace testing;
-        ON_CALL(*mock_buffer_stream, lock_compositor_buffer(_))
+        ON_CALL(*mock_buffer_stream, compositor_acquire(_))
             .WillByDefault(Return(mt::fake_shared(stubbuf)));
     }
     std::shared_ptr<ms::SceneReport> null_scene_report{mr::null_scene_report()};
@@ -149,17 +141,21 @@ struct SurfaceStackCompositor : public testing::Test
     std::shared_ptr<mc::CompositorReport> null_comp_report{mr::null_compositor_report()};
     StubRendererFactory renderer_factory;
     std::chrono::system_clock::time_point timeout;
-    std::shared_ptr<mtd::MockBufferStream> mock_buffer_stream;
+    std::shared_ptr<mtd::MockBufferBundle> mock_buffer_stream;
     std::shared_ptr<ms::BasicSurface> stub_surface;
     ms::SurfaceCreationParameters default_params;
-    BypassStubBuffer stubbuf;
-    CountingDisplayBuffer stub_primary_db;
-    CountingDisplayBuffer stub_secondary_db;
+    mtd::StubBuffer stubbuf;
+    CountingDisplaySyncGroup stub_primary_db;
+    CountingDisplaySyncGroup stub_secondary_db;
     StubDisplay stub_display{stub_primary_db, stub_secondary_db};
+    StubDisplayListener stub_display_listener;
     mc::DefaultDisplayBufferCompositorFactory dbc_factory{
         mt::fake_shared(renderer_factory),
         null_comp_report};
 };
+
+std::chrono::milliseconds const default_delay{-1};
+
 }
 
 TEST_F(SurfaceStackCompositor, composes_on_start_if_told_to_in_constructor)
@@ -168,7 +164,8 @@ TEST_F(SurfaceStackCompositor, composes_on_start_if_told_to_in_constructor)
         mt::fake_shared(stub_display),
         mt::fake_shared(stack),
         mt::fake_shared(dbc_factory),
-        null_comp_report, true);
+        mt::fake_shared(stub_display_listener),
+        null_comp_report, default_delay, true);
     mt_compositor.start();
 
     EXPECT_TRUE(stub_primary_db.has_posted_at_least(1, timeout));
@@ -181,24 +178,26 @@ TEST_F(SurfaceStackCompositor, does_not_composes_on_start_if_told_not_to_in_cons
         mt::fake_shared(stub_display),
         mt::fake_shared(stack),
         mt::fake_shared(dbc_factory),
-        null_comp_report, false);
+        mt::fake_shared(stub_display_listener),
+        null_comp_report, default_delay, false);
     mt_compositor.start();
 
     EXPECT_TRUE(stub_primary_db.has_posted_at_least(0, timeout));
     EXPECT_TRUE(stub_secondary_db.has_posted_at_least(0, timeout));
 }
 
-TEST_F(SurfaceStackCompositor, adding_a_surface_that_has_been_swapped_triggers_a_composition)
+TEST_F(SurfaceStackCompositor, swapping_a_surface_that_has_been_added_triggers_a_composition)
 {
     mc::MultiThreadedCompositor mt_compositor(
         mt::fake_shared(stub_display),
         mt::fake_shared(stack),
         mt::fake_shared(dbc_factory),
-        null_comp_report, false);
+        mt::fake_shared(stub_display_listener),
+        null_comp_report, default_delay, false);
     mt_compositor.start();
 
-    stub_surface->swap_buffers(&stubbuf, [](mg::Buffer*){});
     stack.add_surface(stub_surface, default_params.depth, default_params.input_mode);
+    stub_surface->primary_buffer_stream()->swap_buffers(&stubbuf, [](mg::Buffer*){});
 
     EXPECT_TRUE(stub_primary_db.has_posted_at_least(1, timeout));
     EXPECT_TRUE(stub_secondary_db.has_posted_at_least(1, timeout));
@@ -215,11 +214,12 @@ TEST_F(SurfaceStackCompositor, compositor_runs_until_all_surfaces_buffers_are_co
         mt::fake_shared(stub_display),
         mt::fake_shared(stack),
         mt::fake_shared(dbc_factory),
-        null_comp_report, false);
+        mt::fake_shared(stub_display_listener),
+        null_comp_report, default_delay, false);
     mt_compositor.start();
 
     stack.add_surface(stub_surface, default_params.depth, default_params.input_mode);
-    stub_surface->swap_buffers(&stubbuf, [](mg::Buffer*){});
+    stub_surface->primary_buffer_stream()->swap_buffers(&stubbuf, [](mg::Buffer*){});
 
     EXPECT_TRUE(stub_primary_db.has_posted_at_least(5, timeout));
     EXPECT_TRUE(stub_secondary_db.has_posted_at_least(5, timeout));
@@ -237,11 +237,12 @@ TEST_F(SurfaceStackCompositor, bypassed_compositor_runs_until_all_surfaces_buffe
         mt::fake_shared(stub_display),
         mt::fake_shared(stack),
         mt::fake_shared(dbc_factory),
-        null_comp_report, false);
+        mt::fake_shared(stub_display_listener),
+        null_comp_report, default_delay, false);
     mt_compositor.start();
 
     stack.add_surface(stub_surface, default_params.depth, default_params.input_mode);
-    stub_surface->swap_buffers(&stubbuf, [](mg::Buffer*){});
+    stub_surface->primary_buffer_stream()->swap_buffers(&stubbuf, [](mg::Buffer*){});
 
     EXPECT_TRUE(stub_primary_db.has_posted_at_least(5, timeout));
     EXPECT_TRUE(stub_secondary_db.has_posted_at_least(5, timeout));
@@ -257,11 +258,12 @@ TEST_F(SurfaceStackCompositor, an_empty_scene_retriggers)
         mt::fake_shared(stub_display),
         mt::fake_shared(stack),
         mt::fake_shared(dbc_factory),
-        null_comp_report, false);
+        mt::fake_shared(stub_display_listener),
+        null_comp_report, default_delay, false);
     mt_compositor.start();
 
     stack.add_surface(stub_surface, default_params.depth, default_params.input_mode);
-    stub_surface->swap_buffers(&stubbuf, [](mg::Buffer*){});
+    stub_surface->primary_buffer_stream()->swap_buffers(&stubbuf, [](mg::Buffer*){});
 
     EXPECT_TRUE(stub_primary_db.has_posted_at_least(1, timeout));
     EXPECT_TRUE(stub_secondary_db.has_posted_at_least(1, timeout));
@@ -274,14 +276,15 @@ TEST_F(SurfaceStackCompositor, an_empty_scene_retriggers)
 
 TEST_F(SurfaceStackCompositor, moving_a_surface_triggers_composition)
 {
-    stub_surface->swap_buffers(&stubbuf, [](mg::Buffer*){});
+    stub_surface->primary_buffer_stream()->swap_buffers(&stubbuf, [](mg::Buffer*){});
     stack.add_surface(stub_surface, default_params.depth, default_params.input_mode);
 
     mc::MultiThreadedCompositor mt_compositor(
         mt::fake_shared(stub_display),
         mt::fake_shared(stack),
         mt::fake_shared(dbc_factory),
-        null_comp_report, false);
+        mt::fake_shared(stub_display_listener),
+        null_comp_report, default_delay, false);
 
     mt_compositor.start();
     stub_surface->move_to(geom::Point{1,1});
@@ -292,14 +295,15 @@ TEST_F(SurfaceStackCompositor, moving_a_surface_triggers_composition)
 
 TEST_F(SurfaceStackCompositor, removing_a_surface_triggers_composition)
 {
-    stub_surface->swap_buffers(&stubbuf, [](mg::Buffer*){});
+    stub_surface->primary_buffer_stream()->swap_buffers(&stubbuf, [](mg::Buffer*){});
     stack.add_surface(stub_surface, default_params.depth, default_params.input_mode);
 
     mc::MultiThreadedCompositor mt_compositor(
         mt::fake_shared(stub_display),
         mt::fake_shared(stack),
         mt::fake_shared(dbc_factory),
-        null_comp_report, false);
+        mt::fake_shared(stub_display_listener),
+        null_comp_report, default_delay, false);
 
     mt_compositor.start();
     stack.remove_surface(stub_surface);
@@ -314,16 +318,17 @@ TEST_F(SurfaceStackCompositor, buffer_updates_trigger_composition)
     ON_CALL(*mock_buffer_stream, buffers_ready_for_compositor(_))
         .WillByDefault(testing::Return(1));
     stack.add_surface(stub_surface, default_params.depth, default_params.input_mode);
-    stub_surface->swap_buffers(&stubbuf, [](mg::Buffer*){});
+    stub_surface->primary_buffer_stream()->swap_buffers(&stubbuf, [](mg::Buffer*){});
 
     mc::MultiThreadedCompositor mt_compositor(
         mt::fake_shared(stub_display),
         mt::fake_shared(stack),
         mt::fake_shared(dbc_factory),
-        null_comp_report, false);
+        mt::fake_shared(stub_display_listener),
+        null_comp_report, default_delay, false);
 
     mt_compositor.start();
-    stub_surface->swap_buffers(&stubbuf, [](mg::Buffer*){});
+    stub_surface->primary_buffer_stream()->swap_buffers(&stubbuf, [](mg::Buffer*){});
 
     EXPECT_TRUE(stub_primary_db.has_posted_at_least(1, timeout));
     EXPECT_TRUE(stub_secondary_db.has_posted_at_least(1, timeout));
