@@ -16,7 +16,7 @@
  * Authored by: Alexandros Frantzis <alexandros.frantzis@canonical.com>
  */
 
-#include "mir_test/display_config_matchers.h"
+#include "mir/test/display_config_matchers.h"
 #include "mir/graphics/display_configuration.h"
 #include "mir_protobuf.pb.h"
 #include "mir_toolkit/client_types.h"
@@ -69,7 +69,9 @@ public:
                 protobuf_output.current_mode(),
                 static_cast<MirPixelFormat>(protobuf_output.current_format()),
                 static_cast<MirPowerMode>(protobuf_output.power_mode()),
-                static_cast<MirOrientation>(protobuf_output.orientation())
+                static_cast<MirOrientation>(protobuf_output.orientation()),
+                1.0f,
+                mir_form_factor_monitor
             };
 
             /* Modes */
@@ -135,7 +137,9 @@ public:
                 client_output.current_mode,
                 client_output.current_format,
                 static_cast<MirPowerMode>(client_output.power_mode),
-                static_cast<MirOrientation>(client_output.orientation)
+                static_cast<MirOrientation>(client_output.orientation),
+                1.0f,
+                mir_form_factor_monitor
             };
 
             /* Modes */
@@ -278,6 +282,14 @@ bool mt::compare_display_configurations(mp::DisplayConfiguration const& protobuf
     return compare_display_configurations(config1, display_config);
 }
 
+bool mt::compare_display_configurations(MirDisplayConfiguration const* client_config1,
+                                        MirDisplayConfiguration const* client_config2)
+{
+    TestDisplayConfiguration config1{*client_config1};
+    TestDisplayConfiguration config2{*client_config2};
+    return compare_display_configurations(config1, config2);
+}
+
 bool mt::compare_display_configurations(MirDisplayConfiguration const& client_config,
                                         mp::DisplayConfiguration const& protobuf_config)
 {
@@ -285,3 +297,17 @@ bool mt::compare_display_configurations(MirDisplayConfiguration const& client_co
     TestDisplayConfiguration config2{protobuf_config};
     return compare_display_configurations(config1, config2);
 }
+
+bool mt::compare_display_configurations(graphics::DisplayConfiguration const& display_config1,
+                                        MirDisplayConfiguration const* display_config2)
+{
+    TestDisplayConfiguration config2{*display_config2};
+    return compare_display_configurations(display_config1, config2);
+}
+
+bool mt::compare_display_configurations(MirDisplayConfiguration const* display_config2,
+                                        graphics::DisplayConfiguration const& display_config1)
+{
+    TestDisplayConfiguration config2{*display_config2};
+    return compare_display_configurations(display_config1, config2);
+}
