@@ -38,7 +38,7 @@
 namespace mtf = mir_test_framework;
 
 mtf::UdevEnvironment::UdevEnvironment()
-    : recordings_path(mtf::executable_path() + "/udev_recordings")
+    : recordings_path(mtf::udev_recordings_path())
 {
     testbed = umockdev_testbed_new();
 }
@@ -109,4 +109,41 @@ void mtf::UdevEnvironment::add_standard_device(std::string const& name)
             }
         }
     }
+
+    auto script_filename = recordings_path + "/" + name + ".script";
+    if (stat(script_filename.c_str(), &sb) == 0)
+    {
+        if (S_ISREG(sb.st_mode) || S_ISLNK(sb.st_mode))
+        {
+            if (!umockdev_testbed_load_script(testbed, NULL, script_filename.c_str(), &err))
+            {
+                BOOST_THROW_EXCEPTION(std::runtime_error(std::string("Failed to load device recording: ") +
+                                                         err->message));
+            }
+        }
+    }
+}
+
+void mtf::UdevEnvironment::load_device_ioctls(std::string const& name)
+{
+    auto ioctls_filename = recordings_path + "/" + name + ".ioctl";
+
+    GError* err = nullptr;
+    if (!umockdev_testbed_load_ioctl(testbed, NULL, ioctls_filename.c_str(), &err))
+    {
+        BOOST_THROW_EXCEPTION(std::runtime_error(std::string("Failed to load ioctl recording: ") +
+                                                 err->message));
+    }
+}
+
+void mtf::UdevEnvironment::load_device_evemu(std::string const& name)
+{
+    auto evemu_filename = recordings_path + "/" + name + ".evemu";
+
+    GError* err = nullptr;
+    if (!umockdev_testbed_load_evemu_events(testbed, NULL, evemu_filename.c_str(), &err))
+    {
+        BOOST_THROW_EXCEPTION(std::runtime_error(std::string("Failed to load evemu recording: ") +
+                                                 err->message));
+    }
 }
