@@ -23,12 +23,64 @@
 #include "mir/udev/wrapper.h"
 #include <umockdev.h>
 #include "mir_test_framework/udev_environment.h"
+#include "mir/test/wait_condition.h"
+#include "mir/test/auto_unblock_thread.h"
 
 #include <gtest/gtest.h>
 #include <gmock/gmock.h>
 
+#include <sys/epoll.h>
+
+#include <thread>
+#include <cstring>
+
 namespace mi = mir::input;
 
+TEST(EventHub, does_not_block_on_get_events)
+{
+    mir_test_framework::UdevEnvironment empty_env;
+    auto hub = android::sp<android::EventHub>{new android::EventHub{mir::report::null_input_report()}};
+    auto get_event_terminates = std::make_shared<mir::test::WaitCondition>();
+
+    mir::test::AutoJoinThread reader{
+        [hub,get_event_terminates]
+        {
+            android::RawEvent buffer[10];
+            memset(buffer, 0, sizeof(buffer));
+
+            for (int i = 0; i != 10; ++i)
+                hub->getEvents(buffer, 10);
+            get_event_terminates->wake_up_everyone();
+        }};
+
+    get_event_terminates->wait_for_at_most_seconds(10);
+    EXPECT_TRUE(get_event_terminates->woken());
+}
+
+TEST(EventHub, wakes_on_delay)
+{
+    mir_test_framework::UdevEnvironment empty_env;
+    auto hub = android::sp<android::EventHub>{new android::EventHub{mir::report::null_input_report()}};
+    auto hub_fd = hub->fd();
+    hub->wakeIn(1);
+    auto event_hub_triggered = std::make_shared<mir::test::WaitCondition>();
+
+    mir::test::AutoJoinThread reader{
+        [hub_fd,event_hub_triggered]
+        {
+            struct epoll_event pending_event;
+            std::memset(&pending_event, 0, sizeof pending_event);
+            
+            auto ret = epoll_wait(hub_fd, &pending_event, 1, 3000);
+
+            if (ret > 0 && pending_event.data.u32 == android::EventHub::EPOLL_ID_TIMER)
+                event_hub_triggered->wake_up_everyone();
+        }};
+
+    event_hub_triggered->wait_for_at_most_seconds(4);
+    EXPECT_TRUE(event_hub_triggered->woken());
+}
+
 class EventHubDeviceEnumerationTest : public ::testing::TestWithParam<std::string>
 {
     // Don't actually need any shared state at the moment
@@ -43,7 +95,7 @@ TEST_P(EventHubDeviceEnumerationTest, ScansOnConstruction)
 
     android::RawEvent buffer[10];
     memset(buffer, 0, sizeof(buffer));
-    auto num_events = hub->getEvents(0, buffer, 10);
+    auto num_events = hub->getEvents(buffer, 10);
 
     EXPECT_EQ(static_cast<size_t>(3), num_events);
     EXPECT_EQ(android::EventHub::DEVICE_ADDED, buffer[0].type);
@@ -61,7 +113,7 @@ TEST_P(EventHubDeviceEnumerationTest, GeneratesDeviceAddedOnHotplug)
 
     android::RawEvent buffer[10];
     memset(buffer, 0, sizeof(buffer));
-    auto num_events = hub->getEvents(0, buffer, 10);
+    auto num_events = hub->getEvents(buffer, 10);
 
     EXPECT_EQ(static_cast<size_t>(2), num_events);
     EXPECT_EQ(android::EventHub::DEVICE_ADDED, buffer[0].type);
@@ -71,7 +123,7 @@ TEST_P(EventHubDeviceEnumerationTest, GeneratesDeviceAddedOnHotplug)
     env.add_standard_device(GetParam());
 
     memset(buffer, 0, sizeof(buffer));
-    num_events = hub->getEvents(0, buffer, 10);
+    num_events = hub->getEvents(buffer, 10);
 
     EXPECT_EQ(static_cast<size_t>(2), num_events);
     EXPECT_EQ(android::EventHub::DEVICE_ADDED, buffer[0].type);
@@ -88,7 +140,7 @@ TEST_P(EventHubDeviceEnumerationTest, GeneratesDeviceRemovedOnHotunplug)
 
     android::RawEvent buffer[10];
     // Flush out initial events.
-    auto num_events = hub->getEvents(0, buffer, 10);
+    auto num_events = hub->getEvents(buffer, 10);
 
     mir::udev::Enumerator devices{std::make_shared<mir::udev::Context>()};
     devices.scan_devices();
@@ -108,7 +160,7 @@ TEST_P(EventHubDeviceEnumerationTest, GeneratesDeviceRemovedOnHotunplug)
     }
 
     memset(buffer, 0, sizeof(buffer));
-    num_events = hub->getEvents(0, buffer, 10);
+    num_events = hub->getEvents(buffer, 10);
 
     EXPECT_EQ(static_cast<size_t>(2), num_events);
     EXPECT_EQ(android::EventHub::DEVICE_REMOVED, buffer[0].type);
