@@ -18,10 +18,10 @@
 
 #include "src/platforms/android/server/buffer.h"
 #include "mir/graphics/egl_extensions.h"
-#include "mir_test_doubles/mock_egl.h"
-#include "mir_test_doubles/mock_fence.h"
-#include "mir_test_doubles/mock_android_native_buffer.h"
-#include "mir_test_doubles/mock_android_hw.h"
+#include "mir/test/doubles/mock_egl.h"
+#include "mir/test/doubles/mock_fence.h"
+#include "mir/test/doubles/mock_android_native_buffer.h"
+#include "mir/test/doubles/mock_android_hw.h"
 
 #include <system/window.h>
 #include <stdexcept>
