@@ -18,19 +18,19 @@
 
 #include "src/platforms/android/server/display_buffer.h"
 #include "src/platforms/android/server/gl_context.h"
-#include "src/platforms/android/server/android_format_conversion-inl.h"
-#include "mir_test_doubles/mock_display_device.h"
-#include "mir_test_doubles/mock_display_report.h"
-#include "mir_test_doubles/stub_renderable.h"
-#include "mir_test_doubles/mock_egl.h"
-#include "mir_test_doubles/mock_gl.h"
+#include "src/include/common/mir/graphics/android/android_format_conversion-inl.h"
+#include "mir/test/doubles/mock_display_device.h"
+#include "mir/test/doubles/mock_display_report.h"
+#include "mir/test/doubles/stub_renderable.h"
+#include "mir/test/doubles/mock_egl.h"
+#include "mir/test/doubles/mock_gl.h"
 #include "mir/graphics/android/mir_native_window.h"
-#include "mir_test_doubles/stub_driver_interpreter.h"
-#include "mir_test_doubles/stub_display_buffer.h"
-#include "mir_test_doubles/stub_buffer.h"
-#include "mir_test_doubles/stub_gl_config.h"
-#include "mir_test_doubles/mock_framebuffer_bundle.h"
-#include "mir_test_doubles/stub_gl_program_factory.h"
+#include "mir/test/doubles/stub_driver_interpreter.h"
+#include "mir/test/doubles/stub_display_buffer.h"
+#include "mir/test/doubles/stub_buffer.h"
+#include "mir/test/doubles/stub_gl_config.h"
+#include "mir/test/doubles/mock_framebuffer_bundle.h"
+#include "mir/test/doubles/stub_gl_program_factory.h"
 #include <memory>
 
 namespace geom=mir::geometry;
@@ -64,44 +64,28 @@ struct DisplayBuffer : public ::testing::Test
         std::make_shared<testing::NiceMock<mtd::MockDisplayDevice>>()};
     geom::Size const display_size{433,232};
     double const refresh_rate{60.0};
+    geom::Displacement top_left{0,0};
+    std::unique_ptr<mga::LayerList> list{
+        new mga::LayerList(std::make_shared<mga::IntegerSourceCrop>(), {}, top_left)};
     std::shared_ptr<mtd::MockFBBundle> mock_fb_bundle{
-        std::make_shared<testing::NiceMock<mtd::MockFBBundle>>(
-            display_size, refresh_rate, mir_pixel_format_abgr_8888)};
+        std::make_shared<testing::NiceMock<mtd::MockFBBundle>>(display_size)};
     MirOrientation orientation{mir_orientation_normal};
     mga::DisplayBuffer db{
+        mga::DisplayName::primary,
         std::unique_ptr<mga::LayerList>(
-            new mga::LayerList(std::make_shared<mga::IntegerSourceCrop>(), {})),
+            new mga::LayerList(std::make_shared<mga::IntegerSourceCrop>(), {}, top_left)),
         mock_fb_bundle,
         mock_display_device,
         native_window,
         *gl_context,
         stub_program_factory,
         orientation,
+        top_left,
         mga::OverlayOptimization::enabled};
+
 };
 }
 
-TEST_F(DisplayBuffer, can_post_update_with_gl_only)
-{
-    using namespace testing;
-    std::unique_ptr<mga::LayerList> list(new mga::LayerList(std::make_shared<mga::IntegerSourceCrop>(), {}));
-    EXPECT_CALL(*mock_display_device, commit(
-        mga::DisplayName::primary, Ref(*list), _, _));
-
-    mga::DisplayBuffer db{
-        std::move(list),
-        mock_fb_bundle,
-        mock_display_device,
-        native_window,
-        *gl_context,
-        stub_program_factory,
-        orientation,
-        mga::OverlayOptimization::enabled};
-
-    db.gl_swap_buffers();
-    db.flip();
-}
-
 TEST_F(DisplayBuffer, posts_overlay_list_returns_display_device_decision)
 {
     using namespace testing;
@@ -128,15 +112,15 @@ TEST_F(DisplayBuffer, rotation_transposes_dimensions_and_reports_correctly)
     geom::Size const transposed{display_size.height.as_int(), display_size.width.as_int()};
     EXPECT_EQ(display_size, db.view_area().size);
     EXPECT_EQ(db.orientation(), mir_orientation_normal);
-    db.configure(mir_power_mode_on, mir_orientation_inverted);
+    db.configure(mir_power_mode_on, mir_orientation_inverted, top_left);
 
     EXPECT_EQ(display_size, db.view_area().size);
     EXPECT_EQ(db.orientation(), mir_orientation_inverted);
-    db.configure(mir_power_mode_on, mir_orientation_left);
+    db.configure(mir_power_mode_on, mir_orientation_left, top_left);
 
     EXPECT_EQ(transposed, db.view_area().size);
     EXPECT_EQ(db.orientation(), mir_orientation_left);
-    db.configure(mir_power_mode_on, mir_orientation_right);
+    db.configure(mir_power_mode_on, mir_orientation_right, top_left);
 
     EXPECT_EQ(transposed, db.view_area().size);
     EXPECT_EQ(db.orientation(), mir_orientation_right);
@@ -170,16 +154,21 @@ TEST_F(DisplayBuffer, creates_egl_context_from_shared_context)
     EXPECT_CALL(mock_egl, eglDestroyContext(dummy_display, mock_egl.fake_egl_context))
         .Times(AtLeast(1));
 
+    {
     mga::DisplayBuffer db{
+        mga::DisplayName::primary,
         std::unique_ptr<mga::LayerList>(
-            new mga::LayerList(std::make_shared<mga::IntegerSourceCrop>(), {})),
+            new mga::LayerList(std::make_shared<mga::IntegerSourceCrop>(), {}, top_left)),
         mock_fb_bundle,
         mock_display_device,
         native_window,
         *gl_context,
         stub_program_factory,
         orientation,
+        top_left,
         mga::OverlayOptimization::enabled};
+    }
+    
     testing::Mock::VerifyAndClearExpectations(&mock_egl);
 }
 
@@ -196,27 +185,31 @@ TEST_F(DisplayBuffer, fails_on_egl_resource_creation)
 
     EXPECT_THROW({
         mga::DisplayBuffer db(
+            mga::DisplayName::primary,
             std::unique_ptr<mga::LayerList>(
-                new mga::LayerList(std::make_shared<mga::IntegerSourceCrop>(), {})),
+                new mga::LayerList(std::make_shared<mga::IntegerSourceCrop>(), {}, top_left)),
             mock_fb_bundle,
             mock_display_device,
             native_window,
             *gl_context,
             stub_program_factory,
             orientation,
+            top_left,
             mga::OverlayOptimization::enabled);
     }, std::runtime_error);
 
     EXPECT_THROW({
         mga::DisplayBuffer db(
+            mga::DisplayName::primary,
             std::unique_ptr<mga::LayerList>(
-                new mga::LayerList(std::make_shared<mga::IntegerSourceCrop>(), {})),
+                new mga::LayerList(std::make_shared<mga::IntegerSourceCrop>(), {}, top_left)),
             mock_fb_bundle,
             mock_display_device,
             native_window,
             *gl_context,
             stub_program_factory,
             orientation,
+            top_left,
             mga::OverlayOptimization::enabled);
     }, std::runtime_error);
 }
@@ -246,15 +239,10 @@ TEST_F(DisplayBuffer, notifies_list_that_content_is_cleared)
 {
     EXPECT_CALL(*mock_display_device, content_cleared())
         .Times(3);
-    db.configure(mir_power_mode_off, mir_orientation_normal);
-    db.configure(mir_power_mode_suspend, mir_orientation_normal);
-    db.configure(mir_power_mode_standby, mir_orientation_normal);
-    db.configure(mir_power_mode_on, mir_orientation_normal);
-}
-
-TEST_F(DisplayBuffer, does_not_use_alpha)
-{
-    EXPECT_FALSE(db.uses_alpha());
+    db.configure(mir_power_mode_off, mir_orientation_normal, top_left);
+    db.configure(mir_power_mode_suspend, mir_orientation_normal, top_left);
+    db.configure(mir_power_mode_standby, mir_orientation_normal, top_left);
+    db.configure(mir_power_mode_on, mir_orientation_normal, top_left);
 }
 
 TEST_F(DisplayBuffer, reject_list_if_option_disabled)
@@ -265,14 +253,16 @@ TEST_F(DisplayBuffer, reject_list_if_option_disabled)
 
     mg::RenderableList renderlist{std::make_shared<mtd::StubRenderable>()};
     mga::DisplayBuffer db(
+        mga::DisplayName::primary,
         std::unique_ptr<mga::LayerList>(
-            new mga::LayerList(std::make_shared<mga::IntegerSourceCrop>(), {})),
+            new mga::LayerList(std::make_shared<mga::IntegerSourceCrop>(), {}, top_left)),
         mock_fb_bundle,
         mock_display_device,
         native_window,
         *gl_context,
         stub_program_factory,
         orientation,
+        top_left,
         mga::OverlayOptimization::disabled);
 
     EXPECT_FALSE(db.post_renderables_if_optimizable(renderlist)); 
@@ -287,30 +277,54 @@ TEST_F(DisplayBuffer, rejects_commit_if_list_doesnt_need_commit)
 
     ON_CALL(*mock_display_device, compatible_renderlist(_))
         .WillByDefault(Return(true));
-    ON_CALL(*mock_display_device, commit(_,_,_,_))
-        .WillByDefault(Invoke([](
-            mga::DisplayName,
-            mga::LayerList& list,
-            mga::SwappingGLContext const&,
-            mga::RenderableListCompositor const&)
+    auto set_to_overlays = [](mga::LayerList& list)
+    {
+        auto native_list = list.native_list();
+        for (auto i = 0u; i < native_list->numHwLayers; i++)
         {
-            auto native_list = list.native_list();
-            for (auto i = 0u; i < native_list->numHwLayers; i++)
-            {
-                if (native_list->hwLayers[i].compositionType == HWC_FRAMEBUFFER)
-                    native_list->hwLayers[i].compositionType = HWC_OVERLAY;
-            }
-        }));
+            if (native_list->hwLayers[i].compositionType == HWC_FRAMEBUFFER)
+                native_list->hwLayers[i].compositionType = HWC_OVERLAY;
+        }
+    };
 
     mg::RenderableList renderlist{buffer1, buffer2};
-    EXPECT_TRUE(db.post_renderables_if_optimizable(renderlist)); 
+    EXPECT_TRUE(db.post_renderables_if_optimizable(renderlist));
+    set_to_overlays(db.contents().list);
     EXPECT_FALSE(db.post_renderables_if_optimizable(renderlist)); 
 
     renderlist = mg::RenderableList{buffer2, buffer1}; //ordering changed
     EXPECT_TRUE(db.post_renderables_if_optimizable(renderlist)); 
+    set_to_overlays(db.contents().list);
     EXPECT_FALSE(db.post_renderables_if_optimizable(renderlist)); 
 
     renderlist = mg::RenderableList{buffer3, buffer1}; //buffer changed
     EXPECT_TRUE(db.post_renderables_if_optimizable(renderlist)); 
+    set_to_overlays(db.contents().list);
     EXPECT_FALSE(db.post_renderables_if_optimizable(renderlist)); 
 }
+
+TEST_F(DisplayBuffer, reports_position_correctly)
+{
+    using namespace testing;
+    geom::Point origin;
+    geom::Displacement offset{100, 100};
+
+    EXPECT_THAT(db.view_area().top_left, Eq(origin));
+    db.configure(mir_power_mode_on, orientation, offset);
+    EXPECT_THAT(db.view_area().top_left, Eq(geom::Point{origin + offset}));
+}
+
+//lp: #1485070. Could alternitvely rotate all the renderables, once rotation is supported
+TEST_F(DisplayBuffer, rejects_lists_if_db_is_rotated)
+{
+    ON_CALL(*mock_display_device, compatible_renderlist(testing::_))
+        .WillByDefault(testing::Return(true));
+    mg::RenderableList const renderlist{
+        std::make_shared<mtd::StubRenderable>(),
+        std::make_shared<mtd::StubRenderable>()};
+
+    db.configure(mir_power_mode_on, mir_orientation_inverted, geom::Displacement{0,0});
+    EXPECT_FALSE(db.post_renderables_if_optimizable(renderlist));
+    db.configure(mir_power_mode_on, mir_orientation_normal, geom::Displacement{0,0});
+    EXPECT_TRUE(db.post_renderables_if_optimizable(renderlist));
+}
