@@ -22,19 +22,19 @@
 #include "src/platforms/android/server/hwc_layerlist.h"
 #include "src/platforms/android/server/gl_context.h"
 #include "src/platforms/android/server/hwc_configuration.h"
-#include "mir_test_doubles/mock_android_native_buffer.h"
-#include "mir_test_doubles/stub_renderable.h"
-#include "mir_test_doubles/mock_framebuffer_bundle.h"
-#include "mir_test_doubles/stub_buffer.h"
-#include "mir_test_doubles/mock_hwc_device_wrapper.h"
-#include "mir_test/fake_shared.h"
+#include "mir/test/doubles/mock_android_native_buffer.h"
+#include "mir/test/doubles/stub_renderable.h"
+#include "mir/test/doubles/mock_framebuffer_bundle.h"
+#include "mir/test/doubles/stub_buffer.h"
+#include "mir/test/doubles/mock_hwc_device_wrapper.h"
+#include "mir/test/fake_shared.h"
 #include "hwc_struct_helpers.h"
-#include "mir_test_doubles/mock_swapping_gl_context.h"
-#include "mir_test_doubles/stub_swapping_gl_context.h"
-#include "mir_test_doubles/stub_renderable_list_compositor.h"
-#include "mir_test_doubles/mock_renderable_list_compositor.h"
-#include "mir_test_doubles/mock_renderable.h"
-#include "mir_test_doubles/stub_renderable.h"
+#include "mir/test/doubles/mock_swapping_gl_context.h"
+#include "mir/test/doubles/stub_swapping_gl_context.h"
+#include "mir/test/doubles/stub_renderable_list_compositor.h"
+#include "mir/test/doubles/mock_renderable_list_compositor.h"
+#include "mir/test/doubles/mock_renderable.h"
+#include "mir/test/doubles/stub_renderable.h"
 #include <unistd.h>
 #include <fcntl.h>
 #include <gmock/gmock.h>
@@ -112,10 +112,11 @@ struct HwcDevice : public ::testing::Test
     std::shared_ptr<mtd::StubRenderable> const stub_renderable1;
     std::shared_ptr<mtd::StubRenderable> const stub_renderable2;
     std::shared_ptr<mtd::MockHWCDeviceWrapper> const mock_device;
-    mtd::StubSwappingGLContext const stub_context;
+    mtd::StubSwappingGLContext stub_context;
     mg::RenderableList renderlist;
     std::shared_ptr<mga::LayerAdapter> const layer_adapter;
     mga::DisplayName primary{mga::DisplayName::primary};
+    geom::Displacement offset;
 };
 }
 
@@ -131,9 +132,10 @@ TEST_F(HwcDevice, prepares_a_skip_and_target_layer_by_default)
     EXPECT_CALL(*mock_device, prepare(MatchesPrimaryList(expected_list)))
         .Times(1);
 
-    mga::LayerList list(layer_adapter, {});
+    mga::LayerList list(layer_adapter, {}, geom::Displacement{0,0});
+    mga::DisplayContents content{primary, list, offset, stub_context, stub_compositor};
     mga::HwcDevice device(mock_device);
-    device.commit(primary, list, stub_context, stub_compositor);
+    device.commit({content});
 }
 
 TEST_F(HwcDevice, calls_backup_compositor_when_overlay_rejected)
@@ -163,12 +165,13 @@ TEST_F(HwcDevice, calls_backup_compositor_when_overlay_rejected)
             contents[0]->hwLayers[2].compositionType = HWC_FRAMEBUFFER_TARGET;
         }));
 
-    EXPECT_CALL(mock_compositor, render(expected_renderable_list,Ref(stub_context)))
+    EXPECT_CALL(mock_compositor, render(expected_renderable_list, offset, Ref(stub_context)))
         .InSequence(seq);
 
-    mga::LayerList list(layer_adapter, renderlist);
+    mga::LayerList list(layer_adapter, renderlist, geom::Displacement{});
+    mga::DisplayContents content{primary, list, offset, stub_context, mock_compositor};
     mga::HwcDevice device(mock_device);
-    device.commit(primary, list, stub_context, mock_compositor);
+    device.commit({content});
 }
 
 TEST_F(HwcDevice, swaps_buffers_directly_when_no_renderables)
@@ -179,13 +182,14 @@ TEST_F(HwcDevice, swaps_buffers_directly_when_no_renderables)
     ON_CALL(mock_context, last_rendered_buffer())
         .WillByDefault(Return(stub_fb_buffer));
 
-    EXPECT_CALL(mock_compositor, render(_,_))
+    EXPECT_CALL(mock_compositor, render(_,_,_))
         .Times(0);
     EXPECT_CALL(mock_context, swap_buffers());
 
-    mga::LayerList list(layer_adapter, {});
+    mga::LayerList list(layer_adapter, {}, geom::Displacement{});
+    mga::DisplayContents content{primary, list, offset, mock_context, mock_compositor};
     mga::HwcDevice device(mock_device);
-    device.commit(primary, list, mock_context, mock_compositor);
+    device.commit({content});
 }
 
 TEST_F(HwcDevice, resets_layers_when_prepare_gl_called)
@@ -211,10 +215,11 @@ TEST_F(HwcDevice, resets_layers_when_prepare_gl_called)
         .InSequence(seq);
 
     mga::HwcDevice device(mock_device);
-    mga::LayerList list(layer_adapter, renderlist);
-    device.commit(primary, list, stub_context, stub_compositor);
-    list.update_list({});
-    device.commit(primary, list, stub_context, stub_compositor);
+    mga::LayerList list(layer_adapter, renderlist, geom::Displacement{});
+    mga::DisplayContents content{primary, list, offset, stub_context, stub_compositor};
+    device.commit({content});
+    list.update_list({}, geom::Displacement{});
+    device.commit({content});
 }
 
 TEST_F(HwcDevice, sets_and_updates_fences)
@@ -245,8 +250,9 @@ TEST_F(HwcDevice, sets_and_updates_fences)
         .InSequence(seq);
 
     mga::HwcDevice device(mock_device);
-    mga::LayerList list(layer_adapter, {});
-    device.commit(primary, list, stub_context, stub_compositor);
+    mga::LayerList list(layer_adapter, {}, geom::Displacement{});
+    mga::DisplayContents content{primary, list, offset, stub_context, stub_compositor};
+    device.commit({content});
 
     //check that the retire fence is closed
     bool retire_fence_was_closed{fcntl(hwc_retire_fence, F_GETFD) == -1};
@@ -296,8 +302,9 @@ TEST_F(HwcDevice, commits_correct_list_with_rejected_renderables)
         .InSequence(seq);
 
     mga::HwcDevice device(mock_device);
-    mga::LayerList list(layer_adapter, {stub_renderable1});
-    device.commit(primary, list, stub_context, stub_compositor);
+    mga::LayerList list(layer_adapter, {stub_renderable1}, geom::Displacement{});
+    mga::DisplayContents content{primary, list, offset, stub_context, stub_compositor};
+    device.commit({content});
 }
 
 TEST_F(HwcDevice, commits_correct_list_when_all_accepted_as_overlays)
@@ -359,8 +366,9 @@ TEST_F(HwcDevice, commits_correct_list_when_all_accepted_as_overlays)
         .InSequence(seq);
 
     mga::HwcDevice device(mock_device);
-    mga::LayerList list(layer_adapter, renderlist);
-    device.commit(primary, list, stub_context, stub_compositor);
+    mga::LayerList list(layer_adapter, renderlist, geom::Displacement{});
+    mga::DisplayContents content{primary, list, offset, stub_context, stub_compositor};
+    device.commit({content});
 }
 
 TEST_F(HwcDevice, submits_every_time_if_at_least_one_layer_is_gl_rendered)
@@ -380,10 +388,11 @@ TEST_F(HwcDevice, submits_every_time_if_at_least_one_layer_is_gl_rendered)
     EXPECT_CALL(*mock_device, set(_))
         .Times(2);
 
-    mga::LayerList list(layer_adapter, renderlist);
-    device.commit(primary, list, stub_context, stub_compositor);
-    list.update_list(renderlist);
-    device.commit(primary, list, stub_context, stub_compositor);
+    mga::LayerList list(layer_adapter, renderlist, geom::Displacement{});
+    mga::DisplayContents content{primary, list, offset, stub_context, stub_compositor};
+    device.commit({content});
+    list.update_list(renderlist, geom::Displacement{});
+    device.commit({content});
 }
 
 TEST_F(HwcDevice, resets_composition_type_with_prepare) //lp:1314399
@@ -403,10 +412,11 @@ TEST_F(HwcDevice, resets_composition_type_with_prepare) //lp:1314399
     EXPECT_CALL(*mock_device, prepare(MatchesPrimaryList(expected_list2)))
         .InSequence(seq);
 
-    mga::LayerList list(layer_adapter, renderlist);
-    device.commit(primary, list, stub_context, stub_compositor);
-    list.update_list(renderlist2);
-    device.commit(primary, list, stub_context, stub_compositor);
+    mga::LayerList list(layer_adapter, renderlist, geom::Displacement{});
+    mga::DisplayContents content{primary, list, offset, stub_context, stub_compositor};
+    device.commit({content});
+    list.update_list(renderlist2, geom::Displacement{});
+    device.commit({content});
 }
 
 //note: HWC models overlay layer buffers as owned by the display hardware until a subsequent set.
@@ -420,14 +430,58 @@ TEST_F(HwcDevice, owns_overlay_buffers_until_next_set)
     mga::HwcDevice device(mock_device);
 
     auto use_count_before = stub_buffer1.use_count();
-    mga::LayerList list(layer_adapter, {stub_renderable1});
-    device.commit(primary, list, stub_context, stub_compositor);
+    mga::LayerList list(layer_adapter, {stub_renderable1}, geom::Displacement{});
+    mga::DisplayContents content{primary, list, offset, stub_context, stub_compositor};
+    device.commit({content});
     EXPECT_THAT(stub_buffer1.use_count(), Gt(use_count_before));
-    list.update_list({stub_renderable1});
-    device.commit(primary, list, stub_context, stub_compositor);
+    list.update_list({stub_renderable1}, geom::Displacement{});
+    device.commit({content});
     EXPECT_THAT(stub_buffer1.use_count(), Eq(use_count_before));
 }
 
+TEST_F(HwcDevice, overlays_are_throttled_per_predictive_bypass)
+{
+    using namespace testing;
+    EXPECT_CALL(*mock_device, prepare(_))
+        .WillRepeatedly(Invoke(set_all_layers_to_overlay));
+
+    mga::HwcDevice device(mock_device);
+
+    mga::LayerList list(layer_adapter, {stub_renderable1}, {0,0});
+    mga::DisplayContents content{primary, list, offset, stub_context,
+                                 stub_compositor};
+
+    for (int frame = 0; frame < 5; ++frame)
+    {
+        device.commit({content});
+        ASSERT_THAT(device.recommended_sleep().count(), Ge(8));
+    }
+}
+
+TEST_F(HwcDevice, compositing_disables_predictive_bypass)
+{
+    using namespace testing;
+
+    NiceMock<mtd::MockSwappingGLContext> mock_context;
+    ON_CALL(mock_context, last_rendered_buffer())
+        .WillByDefault(Return(stub_fb_buffer));
+    EXPECT_CALL(mock_context, swap_buffers())
+        .Times(AtLeast(5));
+
+    mga::LayerList list(layer_adapter, {}, geom::Displacement{});
+    mtd::MockRenderableListCompositor mock_compositor;
+    mga::DisplayContents content{primary, list, offset, mock_context,
+                                 mock_compositor};
+
+    mga::HwcDevice device(mock_device);
+    device.commit({content});
+    for (int frame = 0; frame < 5; ++frame)
+    {
+        device.commit({content});
+        ASSERT_EQ(0, device.recommended_sleep().count());
+    }
+}
+
 TEST_F(HwcDevice, does_not_set_acquirefences_when_it_has_set_them_previously_without_update)
 {
     using namespace testing;
@@ -502,13 +556,14 @@ TEST_F(HwcDevice, does_not_set_acquirefences_when_it_has_set_them_previously_wit
         .WillOnce(Invoke(set_fences_fn));
 
     mga::HwcDevice device(mock_device);
-    mga::LayerList list(layer_adapter, renderlist);
-    device.commit(primary, list, stub_context, stub_compositor);
+    mga::LayerList list(layer_adapter, renderlist, geom::Displacement{});
+    mga::DisplayContents content{primary, list, offset, stub_context, stub_compositor};
+    device.commit({content});
     //set only the 2nd layer to a new buffer. the first buffer has the same buffer, and would 
     //still be onscreen if this wasn't against a mock
     stub_renderable1->set_buffer(updated_buffer);
-    list.update_list(renderlist);
-    device.commit(primary, list, stub_context, stub_compositor);
+    list.update_list(renderlist, geom::Displacement{});
+    device.commit({content});
 }
 
 TEST_F(HwcDevice, does_not_own_framebuffer_buffers_past_set)
@@ -526,15 +581,16 @@ TEST_F(HwcDevice, does_not_own_framebuffer_buffers_past_set)
 
     auto use_count_before = stub_buffer1.use_count();
 
-    mga::LayerList list(layer_adapter, {stub_renderable1});
-    device.commit(primary, list, stub_context, stub_compositor);
+    mga::LayerList list(layer_adapter, {stub_renderable1}, geom::Displacement{});
+    mga::DisplayContents content{primary, list, offset, stub_context, stub_compositor};
+    device.commit({content});
     EXPECT_THAT(stub_buffer1.use_count(), Eq(use_count_before));
 }
 
 TEST_F(HwcDevice, rejects_empty_list)
 {
     mga::HwcDevice device(mock_device);
-    std::list<std::shared_ptr<mg::Renderable>> renderlist{};
+    mg::RenderableList renderlist{};
     EXPECT_FALSE(device.compatible_renderlist(renderlist));
 }
 
@@ -565,8 +621,9 @@ TEST_F(HwcDevice, does_not_own_overlay_buffers_after_screen_off)
     mga::HwcDevice device(mock_device);
 
     auto use_count_before = stub_buffer1.use_count();
-    mga::LayerList list(layer_adapter, {stub_renderable1});
-    device.commit(primary, list, stub_context, stub_compositor);
+    mga::LayerList list(layer_adapter, {stub_renderable1}, geom::Displacement{});
+    mga::DisplayContents content{primary, list, offset, stub_context, stub_compositor};
+    device.commit({content});
     EXPECT_THAT(stub_buffer1.use_count(), Gt(use_count_before));
 
     device.content_cleared();
@@ -659,10 +716,11 @@ TEST_F(HwcDevice, tracks_hwc_owned_fences_even_across_list_changes)
     //end second post
 
     mga::HwcDevice device(mock_device);
-    mga::LayerList list(layer_adapter, renderlist1);
-    device.commit(primary, list, stub_context, stub_compositor);
-    list.update_list(renderlist2);
-    device.commit(primary, list, stub_context, stub_compositor);
+    mga::LayerList list(layer_adapter, renderlist1, geom::Displacement{});
+    mga::DisplayContents content{primary, list, offset, stub_context, stub_compositor};
+    device.commit({content});
+    list.update_list(renderlist2, geom::Displacement{});
+    device.commit({content});
 }
 
 TEST_F(HwcDevice, tracks_hwc_owned_fences_across_list_rearrange)
@@ -762,8 +820,45 @@ TEST_F(HwcDevice, tracks_hwc_owned_fences_across_list_rearrange)
     //end second post
 
     mga::HwcDevice device(mock_device);
-    mga::LayerList list(layer_adapter, renderlist);
-    device.commit(primary, list, stub_context, stub_compositor);
-    list.update_list(renderlist2);
-    device.commit(primary, list, stub_context, stub_compositor);
+    mga::LayerList list(layer_adapter, renderlist, geom::Displacement{});
+    mga::DisplayContents content{primary, list, offset, stub_context, stub_compositor};
+    device.commit({content});
+    list.update_list(renderlist2, geom::Displacement{});
+    device.commit({content});
+}
+
+TEST_F(HwcDevice, commits_external_list_with_both_force_gl)
+{
+    using namespace testing;
+    testing::NiceMock<mtd::MockSwappingGLContext> mock_context1;
+    testing::NiceMock<mtd::MockSwappingGLContext> mock_context2;
+    ON_CALL(mock_context1, last_rendered_buffer())
+        .WillByDefault(Return(stub_fb_buffer));
+    ON_CALL(mock_context2, last_rendered_buffer())
+        .WillByDefault(Return(stub_fb_buffer));
+    std::list<hwc_layer_1_t*> expected_list
+    {
+        &skip_layer,
+        &target_layer
+    };
+
+    InSequence seq;
+    EXPECT_CALL(*mock_device, prepare(MatchesLists(expected_list, expected_list)));
+    EXPECT_CALL(mock_context1, make_current());
+    EXPECT_CALL(mock_context1, release_current());
+    EXPECT_CALL(mock_context2, make_current());
+    EXPECT_CALL(mock_context2, release_current());
+    EXPECT_CALL(*mock_device, set(MatchesLists(expected_list, expected_list)));
+
+    mga::LayerList primary_list(layer_adapter, {}, geom::Displacement{});
+    mga::LayerList external_list(layer_adapter, {}, geom::Displacement{});
+
+    mga::HwcDevice device(mock_device);
+
+    mga::DisplayContents primary_content{
+        primary, primary_list, offset, mock_context1, stub_compositor};
+    mga::DisplayContents external_content{
+        mga::DisplayName::external, external_list, offset, mock_context2, stub_compositor};
+
+    device.commit({primary_content, external_content});
 }
