@@ -18,19 +18,19 @@
 
 #include "src/platforms/android/server/hwc_fb_device.h"
 #include "src/platforms/android/server/hwc_configuration.h"
-#include "mir_test_doubles/stub_android_native_buffer.h"
-#include "mir_test_doubles/mock_display_device.h"
-#include "mir_test_doubles/mock_buffer.h"
-#include "mir_test_doubles/mock_android_native_buffer.h"
-#include "mir_test_doubles/mock_framebuffer_bundle.h"
-#include "mir_test_doubles/mock_fb_hal_device.h"
-#include "mir_test_doubles/stub_renderable.h"
-#include "mir_test_doubles/stub_swapping_gl_context.h"
-#include "mir_test_doubles/mock_swapping_gl_context.h"
-#include "mir_test_doubles/mock_egl.h"
-#include "mir_test/auto_unblock_thread.h"
-#include "mir_test_doubles/mock_hwc_device_wrapper.h"
-#include "mir_test_doubles/stub_renderable_list_compositor.h"
+#include "mir/test/doubles/stub_android_native_buffer.h"
+#include "mir/test/doubles/mock_display_device.h"
+#include "mir/test/doubles/mock_buffer.h"
+#include "mir/test/doubles/mock_android_native_buffer.h"
+#include "mir/test/doubles/mock_framebuffer_bundle.h"
+#include "mir/test/doubles/mock_fb_hal_device.h"
+#include "mir/test/doubles/stub_renderable.h"
+#include "mir/test/doubles/stub_swapping_gl_context.h"
+#include "mir/test/doubles/mock_swapping_gl_context.h"
+#include "mir/test/doubles/mock_egl.h"
+#include "mir/test/auto_unblock_thread.h"
+#include "mir/test/doubles/mock_hwc_device_wrapper.h"
+#include "mir/test/doubles/stub_renderable_list_compositor.h"
 #include "src/platforms/android/server/hwc_fallback_gl_renderer.h"
 #include "hwc_struct_helpers.h"
 #include <gtest/gtest.h>
@@ -99,7 +99,7 @@ protected:
     testing::NiceMock<mtd::MockSwappingGLContext> mock_context;
     mtd::StubRenderableListCompositor stub_compositor;
     mga::DisplayName primary{mga::DisplayName::primary};
-    mga::LayerList list{std::make_shared<mga::Hwc10Adapter>(), {}};
+    mga::LayerList list{std::make_shared<mga::Hwc10Adapter>(), {}, geom::Displacement{}};
     hwc_layer_1_t skip_layer;
 };
 }
@@ -122,7 +122,7 @@ TEST_F(HwcFbDevice, hwc10_rejects_overlays)
     mtd::StubRenderableListCompositor stub_compositor;
     auto renderable1 = std::make_shared<mtd::StubRenderable>();
     auto renderable2 = std::make_shared<mtd::StubRenderable>();
-    std::list<std::shared_ptr<mg::Renderable>> renderlist
+    mg::RenderableList renderlist
     {
         renderable1,
         renderable2
@@ -172,5 +172,9 @@ TEST_F(HwcFbDevice, hwc10_post)
     EXPECT_CALL(*mock_fb_device, post_interface(mock_fb_device.get(), &stub_native_buffer->native_handle))
         .InSequence(seq);
 
-    device.commit(primary, list, mock_context, stub_compositor);
+    mga::DisplayContents content{primary, list, geom::Displacement{}, mock_context, stub_compositor};
+    device.commit({content});
+
+    // Predictive bypass not enabled in HwcFbDevice
+    EXPECT_EQ(0, device.recommended_sleep().count());
 }
