@@ -18,10 +18,10 @@
 
 #include "src/server/input/android/android_pointer_controller.h"
 
-#include "mir_test_doubles/mock_input_region.h"
-#include "mir_test_doubles/stub_touch_visualizer.h"
-#include "mir_test/event_factory.h"
-#include "mir_test/fake_shared.h"
+#include "mir/test/doubles/mock_input_region.h"
+#include "mir/test/doubles/stub_touch_visualizer.h"
+#include "mir/test/event_factory.h"
+#include "mir/test/fake_shared.h"
 
 #include <android/input.h>
 
