@@ -16,12 +16,14 @@
  * Authored by: Andreas Pokorny <andreas.pokorny@canonical.com>
  */
 
-#define MIR_INCLUDE_DEPRECATED_EVENT_HEADER
 
 #include "src/server/input/android/input_translator.h"
-#include "mir_test_doubles/mock_input_dispatcher.h"
-#include "mir_test/fake_shared.h"
-#include "mir_test/event_matchers.h"
+#include "mir/events/event_private.h"
+#include "mir/events/event_builders.h"
+
+#include "mir/test/doubles/mock_input_dispatcher.h"
+#include "mir/test/fake_shared.h"
+#include "mir/test/event_matchers.h"
 
 #include "InputListener.h"
 #include "androidfw/Input.h"
@@ -29,6 +31,8 @@
 #include <cstring>
 
 namespace droidinput = android;
+
+namespace mev = mir::events;
 namespace mia = mir::input::android;
 namespace mt = mir::test;
 namespace mtd = mt::doubles;
@@ -49,6 +53,7 @@ public:
     }
     std::chrono::nanoseconds const some_time = std::chrono::nanoseconds(13);
     std::chrono::nanoseconds const later_time = std::chrono::nanoseconds(14);
+    const uint64_t mac = 16;
     const int32_t device_id = 13;
     const uint32_t source_id = 13;
     const uint32_t default_flags = 0;
@@ -63,7 +68,7 @@ public:
     const int32_t meta_state = 0;
     const int32_t edge_flags = 0;
     const int32_t button_state = 0;
-    const int32_t motion_action = mir_motion_action_move;
+    const int32_t motion_action = AMOTION_EVENT_ACTION_MOVE;
     droidinput::PointerCoords coords[MIR_INPUT_EVENT_MAX_POINTER_COUNT];
     droidinput::PointerProperties properties[MIR_INPUT_EVENT_MAX_POINTER_COUNT];
 };
@@ -71,7 +76,6 @@ public:
 struct PolicyFlagTestParameter
 {
     const uint32_t policy_flag;
-    const MirKeyFlag expected_key_flag;
     const unsigned int expected_modifiers;
 };
 
@@ -86,7 +90,7 @@ TEST_F(InputTranslator, notifies_configuration_change)
 {
     using namespace ::testing;
 
-    EXPECT_CALL(dispatcher, configuration_changed(some_time)).Times(1);
+    EXPECT_CALL(dispatcher, dispatch(mt::InputDeviceConfigurationChangedEvent())).Times(1);
 
     droidinput::NotifyConfigurationChangedArgs change(some_time);
     translator.notifyConfigurationChanged(&change);
@@ -96,77 +100,81 @@ TEST_F(InputTranslator, notifies_device_reset)
 {
     using namespace ::testing;
 
-    EXPECT_CALL(dispatcher, device_reset(device_id,later_time)).Times(1);
+    EXPECT_CALL(dispatcher, dispatch(mt::InputDeviceResetEvent())).Times(1);
 
     droidinput::NotifyDeviceResetArgs reset(later_time, device_id);
     translator.notifyDeviceReset(&reset);
 }
 
-TEST_F(InputTranslator, ignores_invalid_key_events)
-{
-    using namespace ::testing;
-
-    EXPECT_CALL(dispatcher, dispatch(_)).Times(0);
-
-    translator.notifyKey(nullptr);
-
-    const int32_t invalid_action = 5;
-    droidinput::NotifyKeyArgs key(some_time, device_id, source_id, default_policy_flags, invalid_action, no_flags,
-                                  arbitrary_key_code, arbitrary_scan_code, no_modifiers, later_time);
-
-    translator.notifyKey(&key);
-}
-
-TEST_F(InputTranslator, ignores_invalid_motion_action)
-{
-    using namespace ::testing;
-
-    EXPECT_CALL(dispatcher, dispatch(_)).Times(0);
-
-    const int32_t invalid_motion_action = 20;
-
-    droidinput::NotifyMotionArgs motion(some_time, device_id, source_id, default_policy_flags, invalid_motion_action,
-                                        no_flags, meta_state, button_state, edge_flags, zero_pointers, properties,
-                                        coords, x_precision, y_precision, later_time);
-
-    translator.notifyMotion(&motion);
-}
-
-TEST_F(InputTranslator, ignores_motion_action_with_wrong_index)
-{
-    using namespace ::testing;
-
-    EXPECT_CALL(dispatcher, dispatch(_)).Times(0);
-
-    const int32_t invalid_motion_action = mir_motion_action_pointer_up | (3 << AMOTION_EVENT_ACTION_POINTER_INDEX_SHIFT);
-
-    droidinput::NotifyMotionArgs motion(some_time, device_id, source_id, default_policy_flags, invalid_motion_action,
-                                        no_flags, meta_state, button_state, edge_flags, zero_pointers, properties,
-                                        coords, x_precision, y_precision, later_time);
-
-    translator.notifyMotion(&motion);
-}
-
 TEST_F(InputTranslator, accepts_motion_action_with_existing_index)
 {
     using namespace ::testing;
 
     EXPECT_CALL(dispatcher, dispatch(_)).Times(1);
 
-    const int32_t valid_motion_action = mir_motion_action_pointer_up | (2 << AMOTION_EVENT_ACTION_POINTER_INDEX_SHIFT);
+    const int32_t valid_motion_action = AMOTION_EVENT_ACTION_POINTER_UP | (2 << AMOTION_EVENT_ACTION_POINTER_INDEX_SHIFT);
     const uint32_t three_pointers = 3;
 
     properties[0].id = 1;
     properties[1].id = 2;
     properties[2].id = 3;
 
-    droidinput::NotifyMotionArgs motion(some_time, device_id, source_id, default_policy_flags, valid_motion_action,
+    droidinput::NotifyMotionArgs motion(some_time, mac, device_id, source_id, 0, valid_motion_action,
                                         no_flags, meta_state, button_state, edge_flags, three_pointers, properties,
                                         coords, x_precision, y_precision, later_time);
 
     translator.notifyMotion(&motion);
 }
 
+MATCHER(EndOfTouchGesture, "")
+{
+    MirEvent const& ev = arg;
+    if (mir_event_get_type(&ev) != mir_event_type_input)
+        return false;
+
+    auto iev = mir_event_get_input_event(&ev);
+    if (mir_input_event_get_type(iev) != mir_input_event_type_touch)
+        return false;
+
+    auto tev = mir_input_event_get_touch_event(iev);
+    if (mir_touch_event_point_count(tev) < 1)
+        return false;
+
+    return mir_touch_event_action(tev, 0) == mir_touch_action_up;
+}
+
+TEST_F(InputTranslator, translates_multifinger_release_correctly)
+{   // Regression test for LP: #1481570
+    using namespace ::testing;
+
+    EXPECT_CALL(dispatcher, dispatch(EndOfTouchGesture()))
+        .Times(1);
+
+    /*
+     * Android often provides old indices on AMOTION_EVENT_ACTION_UP.
+     * But that doesn't matter at all because AMOTION_EVENT_ACTION_UP means
+     * that all fingers were released. (LP: #1481570).
+     */
+    int32_t const invalid_id = 7;
+    int32_t const end_of_gesture = AMOTION_EVENT_ACTION_UP
+                    | (invalid_id << AMOTION_EVENT_ACTION_POINTER_INDEX_SHIFT);
+
+    /*
+     * A multifinger test with only one finger? Yep, that's right. That's
+     * what we see in production. A dummy single finger event used to indicate
+     * all fingers (regardless of number) were released.
+     */
+    properties[0].id = 1;
+    properties[0].toolType = AMOTION_EVENT_TOOL_TYPE_FINGER;
+
+    droidinput::NotifyMotionArgs motion(
+        some_time, mac, device_id, source_id, 0, end_of_gesture,
+        no_flags, meta_state, button_state, edge_flags, 1, properties,
+        coords, x_precision, y_precision, later_time);
+
+    translator.notifyMotion(&motion);
+}
+
 TEST_F(InputTranslator, ignores_motion_with_duplicated_pointerids)
 {
     using namespace ::testing;
@@ -179,7 +187,7 @@ TEST_F(InputTranslator, ignores_motion_with_duplicated_pointerids)
     properties[1].id = 1;
     properties[2].id = 3;
 
-    droidinput::NotifyMotionArgs motion(some_time, device_id, source_id, default_policy_flags, motion_action, no_flags,
+    droidinput::NotifyMotionArgs motion(some_time, mac, device_id, source_id, 0, motion_action, no_flags,
                                         meta_state, button_state, edge_flags, three_pointers, properties, coords,
                                         x_precision, y_precision, later_time);
     translator.notifyMotion(&motion);
@@ -197,7 +205,7 @@ TEST_F(InputTranslator, ignores_motion_with_invalid_pointerids)
     properties[1].id = 1;
     properties[2].id = 3;
 
-    droidinput::NotifyMotionArgs motion(some_time, device_id, source_id, default_policy_flags, motion_action, no_flags,
+    droidinput::NotifyMotionArgs motion(some_time, mac, device_id, source_id, 0, motion_action, no_flags,
                                         meta_state, button_state, edge_flags, three_pointers, properties, coords,
                                         x_precision, y_precision, later_time);
     translator.notifyMotion(&motion);
@@ -209,15 +217,21 @@ TEST_F(InputTranslator, forwards_pointer_positions)
 
     const float x_pos = 12.0f;
     const float y_pos = 30.0f;
-    EXPECT_CALL(dispatcher, dispatch(mt::PointerEventWithPosition(x_pos, y_pos))).Times(1);
+    const float dx = 7.0f;
+    const float dy = 9.3f;
+    EXPECT_CALL(dispatcher, dispatch(AllOf(
+        mt::PointerEventWithPosition(x_pos, y_pos),
+        mt::PointerEventWithDiff(dx, dy)))).Times(1);
 
     const uint32_t one_pointer = 1;
 
     properties[0].id = 23;
     coords[0].setAxisValue(AMOTION_EVENT_AXIS_X, x_pos);
     coords[0].setAxisValue(AMOTION_EVENT_AXIS_Y, y_pos);
+    coords[0].setAxisValue(AMOTION_EVENT_AXIS_RX, dx);
+    coords[0].setAxisValue(AMOTION_EVENT_AXIS_RY, dy);
     
-    droidinput::NotifyMotionArgs motion(some_time, device_id, AINPUT_SOURCE_MOUSE, default_policy_flags, motion_action, no_flags,
+    droidinput::NotifyMotionArgs motion(some_time, mac, device_id, AINPUT_SOURCE_MOUSE, 0, motion_action, no_flags,
                                         meta_state, button_state, edge_flags, one_pointer, properties, coords,
                                         x_precision, y_precision, later_time);
     translator.notifyMotion(&motion);
@@ -231,9 +245,9 @@ TEST_F(InputTranslator, forwards_and_converts_up_down_key_notifications)
     EXPECT_CALL(dispatcher, dispatch(mt::KeyDownEvent())).Times(1);
     EXPECT_CALL(dispatcher, dispatch(mt::KeyUpEvent())).Times(1);
 
-    droidinput::NotifyKeyArgs down(some_time, device_id, source_id, default_policy_flags, mir_key_action_down,
+    droidinput::NotifyKeyArgs down(some_time, mac, device_id, source_id, 0, AKEY_EVENT_ACTION_DOWN,
                                    no_flags, arbitrary_key_code, arbitrary_scan_code, no_modifiers, later_time);
-    droidinput::NotifyKeyArgs up(some_time, device_id, source_id, default_policy_flags, mir_key_action_up,
+    droidinput::NotifyKeyArgs up(some_time, mac, device_id, source_id, 0, AKEY_EVENT_ACTION_UP,
                                  no_flags, arbitrary_key_code, arbitrary_scan_code, no_modifiers, later_time);
 
     translator.notifyKey(&down);
@@ -243,33 +257,29 @@ TEST_F(InputTranslator, forwards_and_converts_up_down_key_notifications)
 TEST_F(InputTranslator, forwards_all_key_event_paramters_correctly)
 {
     using namespace ::testing;
-    MirEvent expected;
-    expected.type = mir_event_type_key;
-    expected.key.event_time = 1;
-    expected.key.device_id = 2;
-    expected.key.source_id = 3;
-    expected.key.action = mir_key_action_down;
-    expected.key.flags = mir_key_flag_long_press;
-    expected.key.scan_code = 4;
-    expected.key.key_code = 5;
-    expected.key.repeat_count = 0;
-    expected.key.down_time = 6;
-    expected.key.modifiers = 7;
-    expected.key.is_system_key = false;
+
+    int32_t const device_id = 2, scan_code = 4, key_code = 5;
+    std::chrono::nanoseconds event_time(1);
+    auto mac = 0;
+
+    auto expected = mev::make_event(MirInputDeviceId(device_id), event_time, mac,
+                                    mir_keyboard_action_down, key_code, scan_code,
+                                    mir_input_event_modifier_shift);
 
     InSequence seq;
-    EXPECT_CALL(dispatcher, dispatch(mt::MirKeyEventMatches(expected))).Times(1);
+    EXPECT_CALL(dispatcher, dispatch(mt::MirKeyEventMatches(*expected))).Times(1);
 
-    droidinput::NotifyKeyArgs notified(std::chrono::nanoseconds(expected.key.event_time),
-                                       expected.key.device_id,
-                                       expected.key.source_id,
+    droidinput::NotifyKeyArgs notified(event_time,
+                                       mac,
+                                       device_id,
+                                       AINPUT_SOURCE_KEYBOARD,
                                        default_policy_flags,
-                                       expected.key.action,
-                                       expected.key.flags,
-                                       expected.key.key_code,
-                                       expected.key.scan_code,
-                                       expected.key.modifiers,
-                                       std::chrono::nanoseconds(expected.key.down_time));
+                                       AKEY_EVENT_ACTION_DOWN,
+                                       0, /* flags */
+                                       key_code,
+                                       scan_code,
+                                       AMETA_SHIFT_ON,
+                                       event_time);
 
     translator.notifyKey(&notified);
 }
@@ -277,68 +287,43 @@ TEST_F(InputTranslator, forwards_all_key_event_paramters_correctly)
 TEST_F(InputTranslator, forwards_all_motion_event_paramters_correctly)
 {
     using namespace ::testing;
-    MirEvent expected;
-    expected.type = mir_event_type_motion;
-    expected.motion.pointer_count = 1;
-    expected.motion.event_time = 2;
-    expected.motion.device_id = 3;
-    expected.motion.source_id = 4;
-    expected.motion.action = mir_motion_action_scroll;
-    expected.motion.flags = mir_motion_flag_window_is_obscured;
-    expected.motion.modifiers = 6;
-    expected.motion.edge_flags = 7;
-    expected.motion.button_state =
-        static_cast<MirMotionButton>(mir_motion_button_forward | mir_motion_button_secondary);
-    expected.motion.x_offset = 0.0f;
-    expected.motion.y_offset = 0.0f;
-    expected.motion.x_precision = 9.0f;
-    expected.motion.y_precision = 10.0f;
-    expected.motion.down_time = 11;
 
-    auto & pointer = expected.motion.pointer_coordinates[0];
-    pointer.id = 1;
-    pointer.x = 12.0f;
-    pointer.raw_x = 12.0f;
-    pointer.y = 13.0f;
-    pointer.raw_y = 13.0f;
-    pointer.touch_major = 14.0f;
-    pointer.touch_minor = 15.0f;
-    pointer.size = 16.0f;
-    pointer.pressure = 17.0f;
-    pointer.orientation = 18.0f;
-    pointer.vscroll = 19.0f;
-    pointer.hscroll = 20.0f;
-    pointer.tool_type = mir_motion_tool_type_finger;
+    std::chrono::nanoseconds event_time(2);
+    auto mac = 0;
+    int32_t device_id = 3;
+    int32_t touch_id = 17;
+    float x = 7, y = 8, pres = 9, tmaj = 10, tmin = 11, size = 12;
 
-    coords[0].setAxisValue(AMOTION_EVENT_AXIS_X, pointer.x);
-    coords[0].setAxisValue(AMOTION_EVENT_AXIS_Y, pointer.y);
-    coords[0].setAxisValue(AMOTION_EVENT_AXIS_TOUCH_MAJOR, pointer.touch_major);
-    coords[0].setAxisValue(AMOTION_EVENT_AXIS_TOUCH_MINOR, pointer.touch_minor);
-    coords[0].setAxisValue(AMOTION_EVENT_AXIS_SIZE, pointer.size);
-    coords[0].setAxisValue(AMOTION_EVENT_AXIS_PRESSURE, pointer.pressure);
-    coords[0].setAxisValue(AMOTION_EVENT_AXIS_ORIENTATION, pointer.orientation);
-    coords[0].setAxisValue(AMOTION_EVENT_AXIS_VSCROLL, pointer.vscroll);
-    coords[0].setAxisValue(AMOTION_EVENT_AXIS_HSCROLL, pointer.hscroll);
-    properties[0].id = pointer.id;
-    properties[0].toolType = pointer.tool_type;
+    auto expected = mev::make_event(MirInputDeviceId(device_id), event_time, mac, mir_input_event_modifier_none);
+    mev::add_touch(*expected,  MirTouchId(touch_id), mir_touch_action_change,
+                   mir_touch_tooltype_finger, x, y, pres, tmaj, tmin, size);
+
+    coords[0].setAxisValue(AMOTION_EVENT_AXIS_X, x);
+    coords[0].setAxisValue(AMOTION_EVENT_AXIS_Y, y);
+    coords[0].setAxisValue(AMOTION_EVENT_AXIS_TOUCH_MAJOR, tmaj);
+    coords[0].setAxisValue(AMOTION_EVENT_AXIS_TOUCH_MINOR, tmin);
+    coords[0].setAxisValue(AMOTION_EVENT_AXIS_SIZE, size);
+    coords[0].setAxisValue(AMOTION_EVENT_AXIS_PRESSURE, pres);
+    properties[0].id = touch_id;
+    properties[0].toolType = AMOTION_EVENT_TOOL_TYPE_FINGER;
     InSequence seq;
-    EXPECT_CALL(dispatcher, dispatch(mt::MirTouchEventMatches(expected))).Times(1);
+    EXPECT_CALL(dispatcher, dispatch(mt::MirTouchEventMatches(*expected))).Times(1);
 
-    droidinput::NotifyMotionArgs notified(std::chrono::nanoseconds(expected.motion.event_time),
-                                          expected.motion.device_id,
-                                          expected.motion.source_id,
+    droidinput::NotifyMotionArgs notified(std::chrono::nanoseconds(event_time),
+                                          mac,
+                                          device_id,
+                                          AINPUT_SOURCE_TOUCHSCREEN,
                                           default_policy_flags,
-                                          expected.motion.action,
-                                          expected.motion.flags,
-                                          expected.motion.modifiers,
-                                          expected.motion.button_state,
-                                          expected.motion.edge_flags,
-                                          expected.motion.pointer_count,
+                                          AMOTION_EVENT_ACTION_HOVER_MOVE,
+                                          0, /* flags */
+                                          0,
+                                          0,
+                                          0, /* edge flags */
+                                          1,
                                           properties,
                                           coords,
-                                          expected.motion.x_precision,
-                                          expected.motion.y_precision,
-                                          std::chrono::nanoseconds(expected.motion.down_time));
+                                          0, 0, /* unused x/y precision */
+                                          event_time);
 
     translator.notifyMotion(&notified);
 }
@@ -353,27 +338,25 @@ TEST_P(InputTranslatorWithPolicyParam, forwards_policy_modifiers_as_flags_and_mo
                     )
                 ).Times(1);
 
-    droidinput::NotifyKeyArgs tester(some_time, device_id, source_id,
-                                     default_policy_flags | GetParam().policy_flag, mir_key_action_down,
+    droidinput::NotifyKeyArgs tester(some_time, mac, device_id, source_id,
+                                     GetParam().policy_flag, AKEY_EVENT_ACTION_DOWN,
                                      no_flags, arbitrary_key_code, arbitrary_scan_code, no_modifiers, later_time);
 
     translator.notifyKey(&tester);
 }
 
-const MirKeyFlag default_key_flag = static_cast<MirKeyFlag>(0);
 INSTANTIATE_TEST_CASE_P(VariousPolicyFlags, InputTranslatorWithPolicyParam,
-                        ::testing::Values(PolicyFlagTestParameter{droidinput::POLICY_FLAG_CAPS_LOCK, default_key_flag,
+                        ::testing::Values(PolicyFlagTestParameter{droidinput::POLICY_FLAG_CAPS_LOCK,
                                                                   mir_input_event_modifier_caps_lock},
-                                          PolicyFlagTestParameter{droidinput::POLICY_FLAG_ALT, default_key_flag,
+                                          PolicyFlagTestParameter{droidinput::POLICY_FLAG_ALT,
                                                                   mir_input_event_modifier_alt | mir_input_event_modifier_alt_left},
-                                          PolicyFlagTestParameter{droidinput::POLICY_FLAG_ALT_GR, default_key_flag,
+                                          PolicyFlagTestParameter{droidinput::POLICY_FLAG_ALT_GR,
                                                                   mir_input_event_modifier_alt | mir_input_event_modifier_alt_right},
-                                          PolicyFlagTestParameter{droidinput::POLICY_FLAG_FUNCTION, default_key_flag,
+                                          PolicyFlagTestParameter{droidinput::POLICY_FLAG_FUNCTION,
                                                                   mir_input_event_modifier_function},
-                                          PolicyFlagTestParameter{droidinput::POLICY_FLAG_SHIFT, default_key_flag,
+                                          PolicyFlagTestParameter{droidinput::POLICY_FLAG_SHIFT,
                                                                   mir_input_event_modifier_shift | mir_input_event_modifier_shift_left},
                                           PolicyFlagTestParameter{droidinput::POLICY_FLAG_VIRTUAL,
-                                                                  mir_key_flag_virtual_hard_key,
                                                                   mir_input_event_modifier_none}));
 
 
