@@ -17,19 +17,22 @@
  */
 
 #include "src/server/scene/surface_stack.h"
+#include "src/server/compositor/buffer_stream_surfaces.h"
 #include "mir/graphics/buffer_properties.h"
 #include "mir/geometry/rectangle.h"
 #include "mir/scene/observer.h"
 #include "mir/scene/surface_creation_parameters.h"
 #include "mir/compositor/scene_element.h"
+#include "mir/compositor/decoration.h"
 #include "src/server/report/null_report_factory.h"
 #include "src/server/scene/basic_surface.h"
 #include "mir/input/input_channel_factory.h"
-#include "mir_test_doubles/stub_input_channel.h"
-#include "mir_test/fake_shared.h"
-#include "mir_test_doubles/stub_buffer_stream.h"
-#include "mir_test_doubles/stub_renderable.h"
-#include "mir_test_doubles/mock_buffer_stream.h"
+#include "mir/test/doubles/stub_input_channel.h"
+#include "mir/test/fake_shared.h"
+#include "mir/test/doubles/stub_buffer_stream.h"
+#include "mir/test/doubles/stub_renderable.h"
+#include "mir/test/doubles/mock_buffer_stream.h"
+#include "mir/test/doubles/mock_buffer_bundle.h"
 
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
@@ -56,7 +59,7 @@ namespace
 void post_a_frame(ms::Surface& s)
 {
     mtd::StubBuffer old_buffer;
-    s.swap_buffers(&old_buffer, [](mg::Buffer*){});
+    s.primary_buffer_stream()->swap_buffers(&old_buffer, [](mg::Buffer*){});
 }
 
 MATCHER_P(SurfaceWithInputReceptionMode, mode, "")
@@ -64,9 +67,14 @@ MATCHER_P(SurfaceWithInputReceptionMode, mode, "")
     return arg->reception_mode() == mode;
 }
 
-MATCHER_P(SceneElementFor, surface, "")
+MATCHER_P(SceneElementForSurface, surface, "")
 {
-    return arg->renderable()->id() == surface.get();
+    return arg->renderable()->id() == surface->primary_buffer_stream().get();
+}
+
+MATCHER_P(SceneElementForStream, stream, "")
+{
+    return arg->renderable()->id() == stream.get();
 }
 
 struct StubInputChannelFactory : public mi::InputChannelFactory
@@ -166,6 +174,7 @@ struct SurfaceStack : public ::testing::Test
             std::shared_ptr<mir::input::InputSender>(),
             std::shared_ptr<mg::CursorImage>(),
             report);
+        invisible_stub_surface->set_hidden(true);
     }
 
     ms::SurfaceCreationParameters default_params;
@@ -207,9 +216,45 @@ TEST_F(SurfaceStack, stacking_order)
     EXPECT_THAT(
         stack.scene_elements_for(compositor_id),
         ElementsAre(
-            SceneElementFor(stub_surface1),
-            SceneElementFor(stub_surface2),
-            SceneElementFor(stub_surface3)));
+            SceneElementForSurface(stub_surface1),
+            SceneElementForSurface(stub_surface2),
+            SceneElementForSurface(stub_surface3)));
+}
+
+TEST_F(SurfaceStack, stacking_order_with_multiple_buffer_streams)
+{
+    using namespace testing;
+    auto stub_stream0 = std::make_shared<mtd::StubBufferStream>();
+    auto stub_stream1 = std::make_shared<mtd::StubBufferStream>();
+    auto stub_stream2 = std::make_shared<mtd::StubBufferStream>();
+    std::list<ms::StreamInfo> streams = {
+        { stub_surface1->buffer_stream(), {0,0} },
+        { stub_stream0, {2,2} },
+        { stub_stream1, {2,3} },
+    };
+    stub_surface1->set_streams(streams);
+
+    streams = {
+        { stub_stream2, {2,4} },
+        { stub_surface3->buffer_stream(), {0,0} }
+    };
+    stub_surface3->set_streams(streams);
+
+    stack.add_surface(stub_surface1, default_params.depth, default_params.input_mode);
+    stack.add_surface(stub_surface2, default_params.depth, default_params.input_mode);
+    stack.add_surface(stub_surface3, default_params.depth, default_params.input_mode);
+
+
+    EXPECT_THAT(
+        stack.scene_elements_for(compositor_id),
+        ElementsAre(
+            SceneElementForSurface(stub_surface1),
+            SceneElementForStream(stub_stream0),
+            SceneElementForStream(stub_stream1),
+            SceneElementForSurface(stub_surface2),
+            SceneElementForStream(stub_stream2),
+            SceneElementForSurface(stub_surface3)
+        ));
 }
 
 TEST_F(SurfaceStack, scene_snapshot_omits_invisible_surfaces)
@@ -223,17 +268,17 @@ TEST_F(SurfaceStack, scene_snapshot_omits_invisible_surfaces)
     EXPECT_THAT(
         stack.scene_elements_for(compositor_id),
         ElementsAre(
-            SceneElementFor(stub_surface1),
-            SceneElementFor(stub_surface2)));
+            SceneElementForSurface(stub_surface1),
+            SceneElementForSurface(stub_surface2)));
 }
 
-TEST_F(SurfaceStack, scene_counts_pending_accurately)
+TEST_F(SurfaceStack, decor_name_is_surface_name)
 {
     using namespace testing;
 
     ms::SurfaceStack stack{report};
     auto surface = std::make_shared<ms::BasicSurface>(
-        std::string("stub"),
+        std::string("Mary had a little lamb"),
         geom::Rectangle{{},{}},
         false,
         std::make_shared<mtd::StubBufferStream>(),
@@ -242,6 +287,80 @@ TEST_F(SurfaceStack, scene_counts_pending_accurately)
         std::shared_ptr<mg::CursorImage>(),
         report);
     stack.add_surface(surface, default_params.depth, default_params.input_mode);
+    surface->configure(mir_surface_attrib_visibility,
+                       mir_surface_visibility_exposed);
+    post_a_frame(*surface);
+
+    auto elements = stack.scene_elements_for(compositor_id);
+    ASSERT_EQ(1, elements.size());
+
+    auto& element = elements.front();
+
+    auto decor = element->decoration();
+    ASSERT_THAT(decor, Ne(nullptr));
+    EXPECT_EQ(mc::Decoration::Type::surface, decor->type);
+    EXPECT_EQ("Mary had a little lamb", decor->name);
+}
+
+TEST_F(SurfaceStack, gets_surface_renames)
+{
+    using namespace testing;
+
+    ms::SurfaceStack stack{report};
+    auto surface = std::make_shared<ms::BasicSurface>(
+        std::string("username@hostname: /"),
+        geom::Rectangle{{},{}},
+        false,
+        std::make_shared<mtd::StubBufferStream>(),
+        std::shared_ptr<mir::input::InputChannel>(),
+        std::shared_ptr<mir::input::InputSender>(),
+        std::shared_ptr<mg::CursorImage>(),
+        report);
+    stack.add_surface(surface, default_params.depth, default_params.input_mode);
+    surface->configure(mir_surface_attrib_visibility,
+                       mir_surface_visibility_exposed);
+    post_a_frame(*surface);
+
+    // (change directory in shell app)
+    surface->rename("username@hostname: ~/Documents");
+
+    auto elements = stack.scene_elements_for(compositor_id);
+    ASSERT_EQ(1, elements.size());
+
+    auto& element = elements.front();
+
+    auto decor = element->decoration();
+    ASSERT_THAT(decor, Ne(nullptr));
+    EXPECT_EQ(mc::Decoration::Type::surface, decor->type);
+    EXPECT_EQ("username@hostname: ~/Documents", decor->name);
+}
+
+TEST_F(SurfaceStack, scene_counts_pending_accurately)
+{
+    using namespace testing;
+    ms::SurfaceStack stack{report};
+    mtd::StubBuffer stub_buffer;
+    int ready = 0;
+    auto mock_queue = std::make_shared<testing::NiceMock<mtd::MockBufferBundle>>();
+    ON_CALL(*mock_queue, buffers_ready_for_compositor(_))
+        .WillByDefault(InvokeWithoutArgs([&]{return ready;}));
+    ON_CALL(*mock_queue, client_release(_))
+        .WillByDefault(InvokeWithoutArgs([&]{ready++;}));
+    ON_CALL(*mock_queue, compositor_acquire(_))
+        .WillByDefault(InvokeWithoutArgs([&]{ready--; return mt::fake_shared(stub_buffer); }));
+ 
+    auto surface = std::make_shared<ms::BasicSurface>(
+        std::string("stub"),
+        geom::Rectangle{{},{}},
+        false,
+        std::make_shared<mc::BufferStreamSurfaces>(mock_queue),
+        std::shared_ptr<mir::input::InputChannel>(),
+        std::shared_ptr<mir::input::InputSender>(),
+        std::shared_ptr<mg::CursorImage>(),
+        report);
+    stack.add_surface(surface, default_params.depth, default_params.input_mode);
+    surface->configure(mir_surface_attrib_visibility,
+                       mir_surface_visibility_exposed);
 
     EXPECT_EQ(0, stack.frames_pending(this));
     post_a_frame(*surface);
@@ -260,6 +379,32 @@ TEST_F(SurfaceStack, scene_counts_pending_accurately)
     }
 }
 
+TEST_F(SurfaceStack, scene_doesnt_count_pending_frames_from_occluded_surfaces)
+{  // Regression test for LP: #1418081
+    using namespace testing;
+
+    ms::SurfaceStack stack{report};
+    auto surface = std::make_shared<ms::BasicSurface>(
+        std::string("stub"),
+        geom::Rectangle{{},{}},
+        false,
+        std::make_shared<mtd::StubBufferStream>(),
+        std::shared_ptr<mir::input::InputChannel>(),
+        std::shared_ptr<mir::input::InputSender>(),
+        std::shared_ptr<mg::CursorImage>(),
+        report);
+
+    stack.add_surface(surface, default_params.depth, default_params.input_mode);
+    surface->configure(mir_surface_attrib_visibility,
+                       mir_surface_visibility_occluded);
+
+    EXPECT_EQ(0, stack.frames_pending(this));
+    post_a_frame(*surface);
+    post_a_frame(*surface);
+    post_a_frame(*surface);
+    EXPECT_EQ(0, stack.frames_pending(this));
+}
+
 TEST_F(SurfaceStack, surfaces_are_emitted_by_layer)
 {
     using namespace testing;
@@ -271,9 +416,9 @@ TEST_F(SurfaceStack, surfaces_are_emitted_by_layer)
     EXPECT_THAT(
         stack.scene_elements_for(compositor_id),
         ElementsAre(
-            SceneElementFor(stub_surface1),
-            SceneElementFor(stub_surface3),
-            SceneElementFor(stub_surface2)));
+            SceneElementForSurface(stub_surface1),
+            SceneElementForSurface(stub_surface3),
+            SceneElementForSurface(stub_surface2)));
 }
 
 
@@ -307,18 +452,18 @@ TEST_F(SurfaceStack, raise_to_top_alters_render_ordering)
     EXPECT_THAT(
         stack.scene_elements_for(compositor_id),
         ElementsAre(
-            SceneElementFor(stub_surface1),
-            SceneElementFor(stub_surface2),
-            SceneElementFor(stub_surface3)));
+            SceneElementForSurface(stub_surface1),
+            SceneElementForSurface(stub_surface2),
+            SceneElementForSurface(stub_surface3)));
 
     stack.raise(stub_surface1);
 
     EXPECT_THAT(
         stack.scene_elements_for(compositor_id),
         ElementsAre(
-            SceneElementFor(stub_surface2),
-            SceneElementFor(stub_surface3),
-            SceneElementFor(stub_surface1)));
+            SceneElementForSurface(stub_surface2),
+            SceneElementForSurface(stub_surface3),
+            SceneElementForSurface(stub_surface1)));
 }
 
 TEST_F(SurfaceStack, depth_id_trumps_raise)
@@ -332,18 +477,18 @@ TEST_F(SurfaceStack, depth_id_trumps_raise)
     EXPECT_THAT(
         stack.scene_elements_for(compositor_id),
         ElementsAre(
-            SceneElementFor(stub_surface1),
-            SceneElementFor(stub_surface2),
-            SceneElementFor(stub_surface3)));
+            SceneElementForSurface(stub_surface1),
+            SceneElementForSurface(stub_surface2),
+            SceneElementForSurface(stub_surface3)));
 
     stack.raise(stub_surface1);
 
     EXPECT_THAT(
         stack.scene_elements_for(compositor_id),
         ElementsAre(
-            SceneElementFor(stub_surface2),
-            SceneElementFor(stub_surface1),
-            SceneElementFor(stub_surface3)));
+            SceneElementForSurface(stub_surface2),
+            SceneElementForSurface(stub_surface1),
+            SceneElementForSurface(stub_surface3)));
 }
 
 TEST_F(SurfaceStack, raise_throw_behavior)
@@ -790,9 +935,9 @@ TEST_F(SurfaceStack, overlays_appear_at_top_of_renderlist)
     EXPECT_THAT(
         stack.scene_elements_for(compositor_id),
         ElementsAre(
-            SceneElementFor(stub_surface1),
-            SceneElementFor(stub_surface2),
-            SceneElementFor(mt::fake_shared(r))));    
+            SceneElementForSurface(stub_surface1),
+            SceneElementForSurface(stub_surface2),
+            SceneElementForStream(mt::fake_shared(r))));    
 }
 
 TEST_F(SurfaceStack, removed_overlays_are_removed)
@@ -808,17 +953,17 @@ TEST_F(SurfaceStack, removed_overlays_are_removed)
     EXPECT_THAT(
         stack.scene_elements_for(compositor_id),
         ElementsAre(
-            SceneElementFor(stub_surface1),
-            SceneElementFor(stub_surface2),
-            SceneElementFor(mt::fake_shared(r))));
+            SceneElementForSurface(stub_surface1),
+            SceneElementForSurface(stub_surface2),
+            SceneElementForStream(mt::fake_shared(r))));
     
     stack.remove_input_visualization(mt::fake_shared(r));
 
     EXPECT_THAT(
         stack.scene_elements_for(compositor_id),
         ElementsAre(
-            SceneElementFor(stub_surface1),
-            SceneElementFor(stub_surface2)));
+            SceneElementForSurface(stub_surface1),
+            SceneElementForSurface(stub_surface2)));
 }
 
 TEST_F(SurfaceStack, scene_observers_notified_of_generic_scene_change)
@@ -854,3 +999,91 @@ TEST_F(SurfaceStack, only_enumerates_exposed_input_surfaces)
     stack.for_each(count_exposed_surfaces);
     EXPECT_THAT(num_exposed_surfaces, Eq(1));
 }
+
+using namespace ::testing;
+
+TEST_F(SurfaceStack, returns_top_surface_under_cursor)
+{
+    geom::Point const cursor_over_all {100, 100};
+    geom::Point const cursor_over_12  {200, 100};
+    geom::Point const cursor_over_1   {600, 600};
+    geom::Point const cursor_over_none{999, 999};
+
+    stack.add_surface(stub_surface1, default_params.depth, default_params.input_mode);
+    stack.add_surface(stub_surface2, default_params.depth, default_params.input_mode);
+    stack.add_surface(stub_surface3, default_params.depth, default_params.input_mode);
+
+    stub_surface1->resize({900, 900});
+    stub_surface2->resize({500, 200});
+    stub_surface3->resize({200, 500});
+
+    EXPECT_THAT(stack.surface_at(cursor_over_all),  Eq(stub_surface3));
+    EXPECT_THAT(stack.surface_at(cursor_over_12),   Eq(stub_surface2));
+    EXPECT_THAT(stack.surface_at(cursor_over_1),    Eq(stub_surface1));
+    EXPECT_THAT(stack.surface_at(cursor_over_none).get(), IsNull());
+}
+
+TEST_F(SurfaceStack, returns_top_visible_surface_under_cursor)
+{
+    geom::Point const cursor_over_all {100, 100};
+    geom::Point const cursor_over_12  {200, 100};
+    geom::Point const cursor_over_1   {600, 600};
+    geom::Point const cursor_over_none{999, 999};
+
+    stack.add_surface(stub_surface1, default_params.depth, default_params.input_mode);
+    stack.add_surface(stub_surface2, default_params.depth, default_params.input_mode);
+    stack.add_surface(stub_surface3, default_params.depth, default_params.input_mode);
+    stack.add_surface(invisible_stub_surface, default_params.depth, default_params.input_mode);
+
+    stub_surface1->resize({900, 900});
+    stub_surface2->resize({500, 200});
+    stub_surface3->resize({200, 500});
+    invisible_stub_surface->resize({999, 999});
+
+    EXPECT_THAT(stack.surface_at(cursor_over_all),  Eq(stub_surface3));
+    EXPECT_THAT(stack.surface_at(cursor_over_12),   Eq(stub_surface2));
+    EXPECT_THAT(stack.surface_at(cursor_over_1),    Eq(stub_surface1));
+    EXPECT_THAT(stack.surface_at(cursor_over_none).get(), IsNull());
+}
+
+TEST_F(SurfaceStack, raise_surfaces_to_top)
+{
+    stack.add_surface(stub_surface1, default_params.depth, default_params.input_mode);
+    stack.add_surface(stub_surface2, default_params.depth, default_params.input_mode);
+    stack.add_surface(stub_surface3, default_params.depth, default_params.input_mode);
+
+    NiceMock<MockSceneObserver> observer;
+    stack.add_observer(mt::fake_shared(observer));
+
+    EXPECT_CALL(observer, surfaces_reordered()).Times(1);
+
+    stack.raise({stub_surface1, stub_surface3});
+    EXPECT_THAT(
+        stack.scene_elements_for(compositor_id),
+        ElementsAre(
+            SceneElementForSurface(stub_surface2),
+            SceneElementForSurface(stub_surface1),
+            SceneElementForSurface(stub_surface3)));
+
+    Mock::VerifyAndClearExpectations(&observer);
+    EXPECT_CALL(observer, surfaces_reordered()).Times(1);
+
+    stack.raise({stub_surface2, stub_surface3});
+    EXPECT_THAT(
+        stack.scene_elements_for(compositor_id),
+        ElementsAre(
+            SceneElementForSurface(stub_surface1),
+            SceneElementForSurface(stub_surface2),
+            SceneElementForSurface(stub_surface3)));
+
+    Mock::VerifyAndClearExpectations(&observer);
+    EXPECT_CALL(observer, surfaces_reordered()).Times(0);
+
+    stack.raise({stub_surface2, stub_surface1, stub_surface3});
+    EXPECT_THAT(
+        stack.scene_elements_for(compositor_id),
+        ElementsAre(
+            SceneElementForSurface(stub_surface1),
+            SceneElementForSurface(stub_surface2),
+            SceneElementForSurface(stub_surface3)));
+}
